# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemMicroscopeConfigurationWidgetBase.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(530, 884)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(Form)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.scrollArea.setFont(font)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 496, 1397))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lineEdit_electron_detector_mode = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_electron_detector_mode.setFont(font)
        self.lineEdit_electron_detector_mode.setObjectName("lineEdit_electron_detector_mode")
        self.gridLayout_2.addWidget(self.lineEdit_electron_detector_mode, 27, 1, 1, 2)
        self.doubleSpinBox_ion_eucentric_height = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_eucentric_height.setFont(font)
        self.doubleSpinBox_ion_eucentric_height.setObjectName("doubleSpinBox_ion_eucentric_height")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_eucentric_height, 32, 1, 1, 2)
        self.label_electron_detector_mode = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_detector_mode.setFont(font)
        self.label_electron_detector_mode.setObjectName("label_electron_detector_mode")
        self.gridLayout_2.addWidget(self.label_electron_detector_mode, 27, 0, 1, 1)
        self.label_configuration_path = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_path.setFont(font)
        self.label_configuration_path.setObjectName("label_configuration_path")
        self.gridLayout_2.addWidget(self.label_configuration_path, 5, 0, 1, 1)
        self.label_ion_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_hfw.setFont(font)
        self.label_ion_hfw.setObjectName("label_ion_hfw")
        self.gridLayout_2.addWidget(self.label_ion_hfw, 36, 0, 1, 1)
        self.label_configuration_name = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_name.setFont(font)
        self.label_configuration_name.setObjectName("label_configuration_name")
        self.gridLayout_2.addWidget(self.label_configuration_name, 4, 0, 1, 1)
        self.label_milling_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_milling_header.setFont(font)
        self.label_milling_header.setObjectName("label_milling_header")
        self.gridLayout_2.addWidget(self.label_milling_header, 47, 0, 1, 3)
        self.doubleSpinBox_electron_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_current.setFont(font)
        self.doubleSpinBox_electron_current.setMaximum(100000000.0)
        self.doubleSpinBox_electron_current.setObjectName("doubleSpinBox_electron_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_current, 23, 1, 1, 2)
        self.spinBox_electron_column_tilt = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_electron_column_tilt.setFont(font)
        self.spinBox_electron_column_tilt.setMaximum(180)
        self.spinBox_electron_column_tilt.setObjectName("spinBox_electron_column_tilt")
        self.gridLayout_2.addWidget(self.spinBox_electron_column_tilt, 17, 1, 1, 2)
        self.checkBox_subsystems_ion_plasma = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_ion_plasma.setFont(font)
        self.checkBox_subsystems_ion_plasma.setObjectName("checkBox_subsystems_ion_plasma")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_ion_plasma, 30, 1, 1, 1)
        self.label_milling_spot_size = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_spot_size.setFont(font)
        self.label_milling_spot_size.setObjectName("label_milling_spot_size")
        self.gridLayout_2.addWidget(self.label_milling_spot_size, 53, 0, 1, 1)
        self.doubleSpinBox_milling_rate = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_rate.setFont(font)
        self.doubleSpinBox_milling_rate.setMaximum(10000000.0)
        self.doubleSpinBox_milling_rate.setObjectName("doubleSpinBox_milling_rate")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_rate, 52, 1, 1, 2)
        self.checkBox_subsystems_stage_tilt = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_stage_tilt.setFont(font)
        self.checkBox_subsystems_stage_tilt.setObjectName("checkBox_subsystems_stage_tilt")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_stage_tilt, 10, 2, 1, 1)
        self.checkBox_imaging_autogamma = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_imaging_autogamma.setFont(font)
        self.checkBox_imaging_autogamma.setObjectName("checkBox_imaging_autogamma")
        self.gridLayout_2.addWidget(self.checkBox_imaging_autogamma, 46, 1, 1, 1)
        self.label_imaging_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_dwell_time.setFont(font)
        self.label_imaging_dwell_time.setObjectName("label_imaging_dwell_time")
        self.gridLayout_2.addWidget(self.label_imaging_dwell_time, 44, 0, 1, 1)
        self.doubleSpinBox_imaging_hfw = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_imaging_hfw.setFont(font)
        self.doubleSpinBox_imaging_hfw.setDecimals(0)
        self.doubleSpinBox_imaging_hfw.setMaximum(100000.0)
        self.doubleSpinBox_imaging_hfw.setObjectName("doubleSpinBox_imaging_hfw")
        self.gridLayout_2.addWidget(self.doubleSpinBox_imaging_hfw, 43, 1, 1, 2)
        self.label_ion_detector_mode = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_detector_mode.setFont(font)
        self.label_ion_detector_mode.setObjectName("label_ion_detector_mode")
        self.gridLayout_2.addWidget(self.label_ion_detector_mode, 38, 0, 1, 1)
        self.lineEdit_configuration_ip_address = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_configuration_ip_address.setFont(font)
        self.lineEdit_configuration_ip_address.setObjectName("lineEdit_configuration_ip_address")
        self.gridLayout_2.addWidget(self.lineEdit_configuration_ip_address, 7, 1, 1, 2)
        self.lineEdit_ion_detector_type = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_ion_detector_type.setFont(font)
        self.lineEdit_ion_detector_type.setObjectName("lineEdit_ion_detector_type")
        self.gridLayout_2.addWidget(self.lineEdit_ion_detector_type, 39, 1, 1, 2)
        self.label_imaging_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_resolution.setFont(font)
        self.label_imaging_resolution.setObjectName("label_imaging_resolution")
        self.gridLayout_2.addWidget(self.label_imaging_resolution, 42, 0, 1, 1)
        self.checkBox_subsystems_stage_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_stage_enabled.setFont(font)
        self.checkBox_subsystems_stage_enabled.setObjectName("checkBox_subsystems_stage_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_stage_enabled, 10, 0, 1, 1)
        self.label_milling_rate = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_rate.setFont(font)
        self.label_milling_rate.setObjectName("label_milling_rate")
        self.gridLayout_2.addWidget(self.label_milling_rate, 52, 0, 1, 1)
        self.label_electron_column_tilt = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_column_tilt.setFont(font)
        self.label_electron_column_tilt.setObjectName("label_electron_column_tilt")
        self.gridLayout_2.addWidget(self.label_electron_column_tilt, 17, 0, 1, 1)
        self.spinBox_stage_shuttle_pre_tilt = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_stage_shuttle_pre_tilt.setFont(font)
        self.spinBox_stage_shuttle_pre_tilt.setMaximum(180)
        self.spinBox_stage_shuttle_pre_tilt.setObjectName("spinBox_stage_shuttle_pre_tilt")
        self.gridLayout_2.addWidget(self.spinBox_stage_shuttle_pre_tilt, 13, 1, 1, 2)
        self.label_electron_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_hfw.setFont(font)
        self.label_electron_hfw.setObjectName("label_electron_hfw")
        self.gridLayout_2.addWidget(self.label_electron_hfw, 25, 0, 1, 1)
        self.spinBox_stage_rotation_180 = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_stage_rotation_180.setFont(font)
        self.spinBox_stage_rotation_180.setMinimum(-360)
        self.spinBox_stage_rotation_180.setMaximum(360)
        self.spinBox_stage_rotation_180.setObjectName("spinBox_stage_rotation_180")
        self.gridLayout_2.addWidget(self.spinBox_stage_rotation_180, 12, 1, 1, 2)
        self.label_ion_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_voltage.setFont(font)
        self.label_ion_voltage.setObjectName("label_ion_voltage")
        self.gridLayout_2.addWidget(self.label_ion_voltage, 33, 0, 1, 1)
        self.doubleSpinBox_milling_voltage = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_voltage.setFont(font)
        self.doubleSpinBox_milling_voltage.setDecimals(0)
        self.doubleSpinBox_milling_voltage.setMaximum(10000000.0)
        self.doubleSpinBox_milling_voltage.setObjectName("doubleSpinBox_milling_voltage")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_voltage, 48, 1, 1, 2)
        self.label_milling_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_dwell_time.setFont(font)
        self.label_milling_dwell_time.setObjectName("label_milling_dwell_time")
        self.gridLayout_2.addWidget(self.label_milling_dwell_time, 51, 0, 1, 1)
        self.label_electron_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_dwell_time.setFont(font)
        self.label_electron_dwell_time.setObjectName("label_electron_dwell_time")
        self.gridLayout_2.addWidget(self.label_electron_dwell_time, 26, 0, 1, 1)
        self.label_electron_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_electron_header.setFont(font)
        self.label_electron_header.setObjectName("label_electron_header")
        self.gridLayout_2.addWidget(self.label_electron_header, 15, 0, 1, 3)
        self.doubleSpinBox_ion_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_current.setFont(font)
        self.doubleSpinBox_ion_current.setMaximum(10000000000.0)
        self.doubleSpinBox_ion_current.setObjectName("doubleSpinBox_ion_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_current, 34, 1, 1, 2)
        self.doubleSpinBox_milling_spot_size = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_spot_size.setFont(font)
        self.doubleSpinBox_milling_spot_size.setMaximum(10000000.0)
        self.doubleSpinBox_milling_spot_size.setObjectName("doubleSpinBox_milling_spot_size")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_spot_size, 53, 1, 1, 2)
        self.spinBox_stage_rotation_reference = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_stage_rotation_reference.setFont(font)
        self.spinBox_stage_rotation_reference.setMinimum(-360)
        self.spinBox_stage_rotation_reference.setMaximum(360)
        self.spinBox_stage_rotation_reference.setObjectName("spinBox_stage_rotation_reference")
        self.gridLayout_2.addWidget(self.spinBox_stage_rotation_reference, 11, 1, 1, 2)
        self.label_configuration_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_configuration_header.setFont(font)
        self.label_configuration_header.setObjectName("label_configuration_header")
        self.gridLayout_2.addWidget(self.label_configuration_header, 3, 0, 1, 1)
        self.doubleSpinBox_ion_voltage = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_voltage.setFont(font)
        self.doubleSpinBox_ion_voltage.setDecimals(0)
        self.doubleSpinBox_ion_voltage.setMaximum(100000000000.0)
        self.doubleSpinBox_ion_voltage.setObjectName("doubleSpinBox_ion_voltage")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_voltage, 33, 1, 1, 2)
        self.label_ion_detector_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_detector_type.setFont(font)
        self.label_ion_detector_type.setObjectName("label_ion_detector_type")
        self.gridLayout_2.addWidget(self.label_ion_detector_type, 39, 0, 1, 1)
        self.label_imaging_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_current.setFont(font)
        self.label_imaging_current.setObjectName("label_imaging_current")
        self.gridLayout_2.addWidget(self.label_imaging_current, 45, 0, 1, 1)
        self.checkBox_imaging_autosave = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_imaging_autosave.setFont(font)
        self.checkBox_imaging_autosave.setObjectName("checkBox_imaging_autosave")
        self.gridLayout_2.addWidget(self.checkBox_imaging_autosave, 46, 2, 1, 1)
        self.label_imaging_beam_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_beam_type.setFont(font)
        self.label_imaging_beam_type.setObjectName("label_imaging_beam_type")
        self.gridLayout_2.addWidget(self.label_imaging_beam_type, 41, 0, 1, 1)
        self.checkBox_imaging_autocontrast = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_imaging_autocontrast.setFont(font)
        self.checkBox_imaging_autocontrast.setObjectName("checkBox_imaging_autocontrast")
        self.gridLayout_2.addWidget(self.checkBox_imaging_autocontrast, 46, 0, 1, 1)
        self.checkBox_subsystems_electron_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_electron_enabled.setFont(font)
        self.checkBox_subsystems_electron_enabled.setObjectName("checkBox_subsystems_electron_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_electron_enabled, 16, 0, 1, 1)
        self.label_stage_rotation_180 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_rotation_180.setFont(font)
        self.label_stage_rotation_180.setObjectName("label_stage_rotation_180")
        self.gridLayout_2.addWidget(self.label_stage_rotation_180, 12, 0, 1, 1)
        self.label_imaging_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_hfw.setFont(font)
        self.label_imaging_hfw.setObjectName("label_imaging_hfw")
        self.gridLayout_2.addWidget(self.label_imaging_hfw, 43, 0, 1, 1)
        self.doubleSpinBox_electron_voltage = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_voltage.setFont(font)
        self.doubleSpinBox_electron_voltage.setDecimals(0)
        self.doubleSpinBox_electron_voltage.setMaximum(100000000000.0)
        self.doubleSpinBox_electron_voltage.setObjectName("doubleSpinBox_electron_voltage")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_voltage, 22, 1, 1, 2)
        self.label_configuration_ip_address = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_ip_address.setFont(font)
        self.label_configuration_ip_address.setObjectName("label_configuration_ip_address")
        self.gridLayout_2.addWidget(self.label_configuration_ip_address, 7, 0, 1, 1)
        self.lineEdit_electron_detector_type = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_electron_detector_type.setFont(font)
        self.lineEdit_electron_detector_type.setObjectName("lineEdit_electron_detector_type")
        self.gridLayout_2.addWidget(self.lineEdit_electron_detector_type, 28, 1, 1, 2)
        self.checkBox_subsystems_manipulator_rotation = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_manipulator_rotation.setFont(font)
        self.checkBox_subsystems_manipulator_rotation.setObjectName("checkBox_subsystems_manipulator_rotation")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_manipulator_rotation, 59, 1, 1, 1)
        self.lineEdit_configuration_path = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_configuration_path.setFont(font)
        self.lineEdit_configuration_path.setObjectName("lineEdit_configuration_path")
        self.gridLayout_2.addWidget(self.lineEdit_configuration_path, 5, 1, 1, 2)
        self.label_stage_rotation_reference = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_rotation_reference.setFont(font)
        self.label_stage_rotation_reference.setObjectName("label_stage_rotation_reference")
        self.gridLayout_2.addWidget(self.label_stage_rotation_reference, 11, 0, 1, 1)
        self.label_ion_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_ion_header.setFont(font)
        self.label_ion_header.setObjectName("label_ion_header")
        self.gridLayout_2.addWidget(self.label_ion_header, 29, 0, 1, 3)
        self.label_configuration_manufacturer = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_manufacturer.setFont(font)
        self.label_configuration_manufacturer.setObjectName("label_configuration_manufacturer")
        self.gridLayout_2.addWidget(self.label_configuration_manufacturer, 6, 0, 1, 1)
        self.label_imaging_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_imaging_header.setFont(font)
        self.label_imaging_header.setObjectName("label_imaging_header")
        self.gridLayout_2.addWidget(self.label_imaging_header, 40, 0, 1, 3)
        self.comboBox_configuration_manufacturer = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.comboBox_configuration_manufacturer.setFont(font)
        self.comboBox_configuration_manufacturer.setObjectName("comboBox_configuration_manufacturer")
        self.gridLayout_2.addWidget(self.comboBox_configuration_manufacturer, 6, 1, 1, 2)
        self.checkBox_subsystems_gis_sputter_coater_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_gis_sputter_coater_enabled.setFont(font)
        self.checkBox_subsystems_gis_sputter_coater_enabled.setObjectName("checkBox_subsystems_gis_sputter_coater_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_gis_sputter_coater_enabled, 60, 2, 1, 1)
        self.doubleSpinBox_milling_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_current.setFont(font)
        self.doubleSpinBox_milling_current.setMaximum(100000000.0)
        self.doubleSpinBox_milling_current.setObjectName("doubleSpinBox_milling_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_current, 49, 1, 1, 2)
        self.spinBox_ion_column_tilt = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_ion_column_tilt.setFont(font)
        self.spinBox_ion_column_tilt.setMaximum(180)
        self.spinBox_ion_column_tilt.setObjectName("spinBox_ion_column_tilt")
        self.gridLayout_2.addWidget(self.spinBox_ion_column_tilt, 31, 1, 1, 2)
        self.label_electron_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_current.setFont(font)
        self.label_electron_current.setObjectName("label_electron_current")
        self.gridLayout_2.addWidget(self.label_electron_current, 23, 0, 1, 1)
        self.checkBox_subsystems_stage_rotation = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_stage_rotation.setFont(font)
        self.checkBox_subsystems_stage_rotation.setObjectName("checkBox_subsystems_stage_rotation")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_stage_rotation, 10, 1, 1, 1)
        self.checkBox_subsystems_gis_multichem_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_gis_multichem_enabled.setFont(font)
        self.checkBox_subsystems_gis_multichem_enabled.setObjectName("checkBox_subsystems_gis_multichem_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_gis_multichem_enabled, 60, 1, 1, 1)
        self.label_milling_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_voltage.setFont(font)
        self.label_milling_voltage.setObjectName("label_milling_voltage")
        self.gridLayout_2.addWidget(self.label_milling_voltage, 48, 0, 1, 1)
        self.checkBox_subsystems_ion_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_ion_enabled.setFont(font)
        self.checkBox_subsystems_ion_enabled.setObjectName("checkBox_subsystems_ion_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_ion_enabled, 30, 0, 1, 1)
        self.checkBox_subsystems_manipulator_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_manipulator_enabled.setFont(font)
        self.checkBox_subsystems_manipulator_enabled.setObjectName("checkBox_subsystems_manipulator_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_manipulator_enabled, 59, 0, 1, 1)
        self.label_stage_shuttle_pre_tilt = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_shuttle_pre_tilt.setFont(font)
        self.label_stage_shuttle_pre_tilt.setObjectName("label_stage_shuttle_pre_tilt")
        self.gridLayout_2.addWidget(self.label_stage_shuttle_pre_tilt, 13, 0, 1, 1)
        self.doubleSpinBox_electron_eucentric_height = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_eucentric_height.setFont(font)
        self.doubleSpinBox_electron_eucentric_height.setObjectName("doubleSpinBox_electron_eucentric_height")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_eucentric_height, 21, 1, 1, 2)
        self.label_electron_eucentric_height = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_eucentric_height.setFont(font)
        self.label_electron_eucentric_height.setObjectName("label_electron_eucentric_height")
        self.gridLayout_2.addWidget(self.label_electron_eucentric_height, 21, 0, 1, 1)
        self.label_stage_manipulator_height_limit = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_manipulator_height_limit.setFont(font)
        self.label_stage_manipulator_height_limit.setObjectName("label_stage_manipulator_height_limit")
        self.gridLayout_2.addWidget(self.label_stage_manipulator_height_limit, 14, 0, 1, 1)
        self.checkBox_subsystems_gis_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_gis_enabled.setFont(font)
        self.checkBox_subsystems_gis_enabled.setObjectName("checkBox_subsystems_gis_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_gis_enabled, 60, 0, 1, 1)
        self.label_electron_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_voltage.setFont(font)
        self.label_electron_voltage.setObjectName("label_electron_voltage")
        self.gridLayout_2.addWidget(self.label_electron_voltage, 22, 0, 1, 1)
        self.label_ion_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_resolution.setFont(font)
        self.label_ion_resolution.setObjectName("label_ion_resolution")
        self.gridLayout_2.addWidget(self.label_ion_resolution, 35, 0, 1, 1)
        self.doubleSpinBox_milling_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_dwell_time.setFont(font)
        self.doubleSpinBox_milling_dwell_time.setMaximum(1000000.0)
        self.doubleSpinBox_milling_dwell_time.setObjectName("doubleSpinBox_milling_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_dwell_time, 51, 1, 1, 2)
        self.label_ion_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_current.setFont(font)
        self.label_ion_current.setObjectName("label_ion_current")
        self.gridLayout_2.addWidget(self.label_ion_current, 34, 0, 1, 1)
        self.label_ion_column_tilt = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_column_tilt.setFont(font)
        self.label_ion_column_tilt.setObjectName("label_ion_column_tilt")
        self.gridLayout_2.addWidget(self.label_ion_column_tilt, 31, 0, 1, 1)
        self.lineEdit_milling_preset = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_milling_preset.setFont(font)
        self.lineEdit_milling_preset.setObjectName("lineEdit_milling_preset")
        self.gridLayout_2.addWidget(self.lineEdit_milling_preset, 54, 1, 1, 2)
        self.lineEdit_ion_detector_mode = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_ion_detector_mode.setFont(font)
        self.lineEdit_ion_detector_mode.setObjectName("lineEdit_ion_detector_mode")
        self.gridLayout_2.addWidget(self.lineEdit_ion_detector_mode, 38, 1, 1, 2)
        self.label_ion_eucentric_height = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_eucentric_height.setFont(font)
        self.label_ion_eucentric_height.setObjectName("label_ion_eucentric_height")
        self.gridLayout_2.addWidget(self.label_ion_eucentric_height, 32, 0, 1, 1)
        self.doubleSpinBox_imaging_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_imaging_dwell_time.setFont(font)
        self.doubleSpinBox_imaging_dwell_time.setMaximum(1000000.0)
        self.doubleSpinBox_imaging_dwell_time.setObjectName("doubleSpinBox_imaging_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_imaging_dwell_time, 44, 1, 1, 2)
        self.comboBox_imaging_beam_type = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.comboBox_imaging_beam_type.setFont(font)
        self.comboBox_imaging_beam_type.setObjectName("comboBox_imaging_beam_type")
        self.gridLayout_2.addWidget(self.comboBox_imaging_beam_type, 41, 1, 1, 2)
        self.doubleSpinBox_imaging_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_imaging_current.setFont(font)
        self.doubleSpinBox_imaging_current.setMaximum(10000000.0)
        self.doubleSpinBox_imaging_current.setObjectName("doubleSpinBox_imaging_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_imaging_current, 45, 1, 1, 2)
        self.label_milling_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_current.setFont(font)
        self.label_milling_current.setObjectName("label_milling_current")
        self.gridLayout_2.addWidget(self.label_milling_current, 49, 0, 1, 1)
        self.label_subsystems_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_subsystems_header.setFont(font)
        self.label_subsystems_header.setObjectName("label_subsystems_header")
        self.gridLayout_2.addWidget(self.label_subsystems_header, 55, 0, 1, 3)
        self.label_stage_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_stage_header.setFont(font)
        self.label_stage_header.setObjectName("label_stage_header")
        self.gridLayout_2.addWidget(self.label_stage_header, 9, 0, 1, 3)
        self.spinBox_imaging_resolution_x = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_imaging_resolution_x.setFont(font)
        self.spinBox_imaging_resolution_x.setMaximum(100000)
        self.spinBox_imaging_resolution_x.setObjectName("spinBox_imaging_resolution_x")
        self.gridLayout_2.addWidget(self.spinBox_imaging_resolution_x, 42, 1, 1, 1)
        self.label_electron_detector_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_detector_type.setFont(font)
        self.label_electron_detector_type.setObjectName("label_electron_detector_type")
        self.gridLayout_2.addWidget(self.label_electron_detector_type, 28, 0, 1, 1)
        self.label_electron_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_resolution.setFont(font)
        self.label_electron_resolution.setObjectName("label_electron_resolution")
        self.gridLayout_2.addWidget(self.label_electron_resolution, 24, 0, 1, 1)
        self.lineEdit_configuration_name = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_configuration_name.setFont(font)
        self.lineEdit_configuration_name.setObjectName("lineEdit_configuration_name")
        self.gridLayout_2.addWidget(self.lineEdit_configuration_name, 4, 1, 1, 2)
        self.spinBox_imaging_resolution_y = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_imaging_resolution_y.setFont(font)
        self.spinBox_imaging_resolution_y.setMaximum(100000)
        self.spinBox_imaging_resolution_y.setObjectName("spinBox_imaging_resolution_y")
        self.gridLayout_2.addWidget(self.spinBox_imaging_resolution_y, 42, 2, 1, 1)
        self.label_milling_preset = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_preset.setFont(font)
        self.label_milling_preset.setObjectName("label_milling_preset")
        self.gridLayout_2.addWidget(self.label_milling_preset, 54, 0, 1, 1)
        self.doubleSpinBox_stage_manipulator_height_limit = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_stage_manipulator_height_limit.setFont(font)
        self.doubleSpinBox_stage_manipulator_height_limit.setObjectName("doubleSpinBox_stage_manipulator_height_limit")
        self.gridLayout_2.addWidget(self.doubleSpinBox_stage_manipulator_height_limit, 14, 1, 1, 2)
        self.checkBox_subsystems_manipulator_tilt = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_manipulator_tilt.setFont(font)
        self.checkBox_subsystems_manipulator_tilt.setObjectName("checkBox_subsystems_manipulator_tilt")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_manipulator_tilt, 59, 2, 1, 1)
        self.label_ion_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_dwell_time.setFont(font)
        self.label_ion_dwell_time.setObjectName("label_ion_dwell_time")
        self.gridLayout_2.addWidget(self.label_ion_dwell_time, 37, 0, 1, 1)
        self.doubleSpinBox_ion_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_dwell_time.setFont(font)
        self.doubleSpinBox_ion_dwell_time.setMaximum(1000000.0)
        self.doubleSpinBox_ion_dwell_time.setObjectName("doubleSpinBox_ion_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_dwell_time, 37, 1, 1, 2)
        self.doubleSpinBox_electron_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_dwell_time.setFont(font)
        self.doubleSpinBox_electron_dwell_time.setMaximum(10000000.0)
        self.doubleSpinBox_electron_dwell_time.setObjectName("doubleSpinBox_electron_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_dwell_time, 26, 1, 1, 2)
        self.doubleSpinBox_electron_hfw = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_hfw.setFont(font)
        self.doubleSpinBox_electron_hfw.setDecimals(0)
        self.doubleSpinBox_electron_hfw.setMaximum(100000.0)
        self.doubleSpinBox_electron_hfw.setObjectName("doubleSpinBox_electron_hfw")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_hfw, 25, 1, 1, 2)
        self.doubleSpinBox_ion_hfw = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_hfw.setFont(font)
        self.doubleSpinBox_ion_hfw.setDecimals(0)
        self.doubleSpinBox_ion_hfw.setMaximum(100000.0)
        self.doubleSpinBox_ion_hfw.setObjectName("doubleSpinBox_ion_hfw")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_hfw, 36, 1, 1, 2)
        self.spinBox_electron_resolution_x = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_electron_resolution_x.setFont(font)
        self.spinBox_electron_resolution_x.setMaximum(100000)
        self.spinBox_electron_resolution_x.setObjectName("spinBox_electron_resolution_x")
        self.gridLayout_2.addWidget(self.spinBox_electron_resolution_x, 24, 1, 1, 1)
        self.spinBox_electron_resolution_y = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_electron_resolution_y.setFont(font)
        self.spinBox_electron_resolution_y.setMaximum(100000)
        self.spinBox_electron_resolution_y.setObjectName("spinBox_electron_resolution_y")
        self.gridLayout_2.addWidget(self.spinBox_electron_resolution_y, 24, 2, 1, 1)
        self.spinBox_ion_resolution_x = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_ion_resolution_x.setFont(font)
        self.spinBox_ion_resolution_x.setMaximum(100000000)
        self.spinBox_ion_resolution_x.setObjectName("spinBox_ion_resolution_x")
        self.gridLayout_2.addWidget(self.spinBox_ion_resolution_x, 35, 1, 1, 1)
        self.spinBox_ion_resolution_y = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_ion_resolution_y.setFont(font)
        self.spinBox_ion_resolution_y.setMaximum(100000000)
        self.spinBox_ion_resolution_y.setObjectName("spinBox_ion_resolution_y")
        self.gridLayout_2.addWidget(self.spinBox_ion_resolution_y, 35, 2, 1, 1)
        self.checkBox_show_advanced_settings = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_show_advanced_settings.setFont(font)
        self.checkBox_show_advanced_settings.setObjectName("checkBox_show_advanced_settings")
        self.gridLayout_2.addWidget(self.checkBox_show_advanced_settings, 0, 1, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 2, 0, 1, 2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_electron_detector_mode.setText(_translate("Form", "Detector Mode"))
        self.label_configuration_path.setText(_translate("Form", "Path"))
        self.label_ion_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_configuration_name.setText(_translate("Form", "Name"))
        self.label_milling_header.setText(_translate("Form", "Milling"))
        self.checkBox_subsystems_ion_plasma.setText(_translate("Form", "Ion Beam Plasma"))
        self.label_milling_spot_size.setText(_translate("Form", "Spot Size (nm)"))
        self.checkBox_subsystems_stage_tilt.setText(_translate("Form", "Stage Tilt"))
        self.checkBox_imaging_autogamma.setText(_translate("Form", "AutoGamma"))
        self.label_imaging_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.label_ion_detector_mode.setText(_translate("Form", "Detector Mode"))
        self.label_imaging_resolution.setText(_translate("Form", "Resolution"))
        self.checkBox_subsystems_stage_enabled.setText(_translate("Form", "Stage Enabled"))
        self.label_milling_rate.setText(_translate("Form", "Rate (um3/s)"))
        self.label_electron_column_tilt.setText(_translate("Form", "Column Tilt (deg)"))
        self.label_electron_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_ion_voltage.setText(_translate("Form", "Voltage (V)"))
        self.label_milling_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.label_electron_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.label_electron_header.setText(_translate("Form", "Electron Beam"))
        self.label_configuration_header.setText(_translate("Form", "Core"))
        self.label_ion_detector_type.setText(_translate("Form", "Detector Type"))
        self.label_imaging_current.setText(_translate("Form", "Imaging Current (nA)"))
        self.checkBox_imaging_autosave.setText(_translate("Form", "AutoSave"))
        self.label_imaging_beam_type.setText(_translate("Form", "Beam Type"))
        self.checkBox_imaging_autocontrast.setText(_translate("Form", "AutoContrast"))
        self.checkBox_subsystems_electron_enabled.setText(_translate("Form", "Electron Beam Enabled"))
        self.label_stage_rotation_180.setText(_translate("Form", "Rotation 180 (deg)"))
        self.label_imaging_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_configuration_ip_address.setText(_translate("Form", "IP Address"))
        self.checkBox_subsystems_manipulator_rotation.setText(_translate("Form", "Manipulator Rotation"))
        self.label_stage_rotation_reference.setText(_translate("Form", "Rotation Reference (deg)"))
        self.label_ion_header.setText(_translate("Form", "Ion Beam"))
        self.label_configuration_manufacturer.setText(_translate("Form", "Manufacturer"))
        self.label_imaging_header.setText(_translate("Form", "Imaging"))
        self.checkBox_subsystems_gis_sputter_coater_enabled.setText(_translate("Form", "Sputter Coat Enabled"))
        self.label_electron_current.setText(_translate("Form", "Current (nA)"))
        self.checkBox_subsystems_stage_rotation.setText(_translate("Form", "Stage Rotation"))
        self.checkBox_subsystems_gis_multichem_enabled.setText(_translate("Form", "MultiChem Enabled"))
        self.label_milling_voltage.setText(_translate("Form", "Milling Voltage (V)"))
        self.checkBox_subsystems_ion_enabled.setText(_translate("Form", "Ion Beam Enabled"))
        self.checkBox_subsystems_manipulator_enabled.setText(_translate("Form", "Manipulator Enabled"))
        self.label_stage_shuttle_pre_tilt.setText(_translate("Form", "Shuttle Pre-Tilt (deg)"))
        self.label_electron_eucentric_height.setText(_translate("Form", "Eucentric Height (mm)"))
        self.label_stage_manipulator_height_limit.setText(_translate("Form", "Manipulator Height Limit (mm)"))
        self.checkBox_subsystems_gis_enabled.setText(_translate("Form", "GIS Enabled"))
        self.label_electron_voltage.setText(_translate("Form", "Voltage (V)"))
        self.label_ion_resolution.setText(_translate("Form", "Resolution"))
        self.label_ion_current.setText(_translate("Form", "Current (nA)"))
        self.label_ion_column_tilt.setText(_translate("Form", "Column Tilt (deg)"))
        self.label_ion_eucentric_height.setText(_translate("Form", "Eucentric Height (mm)"))
        self.label_milling_current.setText(_translate("Form", "Milling Current (nA)"))
        self.label_subsystems_header.setText(_translate("Form", "Subsystems"))
        self.label_stage_header.setText(_translate("Form", "Stage"))
        self.label_electron_detector_type.setText(_translate("Form", "Detector Type"))
        self.label_electron_resolution.setText(_translate("Form", "Resolution"))
        self.label_milling_preset.setText(_translate("Form", "Preset "))
        self.checkBox_subsystems_manipulator_tilt.setText(_translate("Form", "Manipulator Tilt"))
        self.label_ion_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.checkBox_show_advanced_settings.setText(_translate("Form", "Show Advanced Settings"))



import streamlit as st

from fibsem.tools import _parser

import plotly.express as px


st.set_page_config(layout="wide", page_title="OpenFIBSEM Telemetry")
st.title("OpenFIBSEM Telemetry")


DEFAULT_PATH = "data2.csv"

PATH = st.text_input("Path to telemetry data", DEFAULT_PATH)

@st.cache_data
def _load_data(PATH):
    return _parser._parse_health_monitor_data(PATH) 

df_data = _load_data(PATH)



tabs = st.tabs(["Raw Data", "Filter Data", "Beam On/Off"])

with tabs[0]:
    # st.write(df_data)
    st.write("### RAW DATA ###")

with tabs[1]:

    # FILTER BY SUBSYSTEM
    st.write("### FILTER BY SUBSYSTEM ###")

    subsystems = df_data.columns.str.split(".").str[0].unique().tolist()
    subsystem = st.selectbox("Select Subsystem", subsystems)
    # also keep datetime column
    subsystem_cols =  ["datetime"] + [col for col in df_data.columns if subsystem in col]


    # TODO: filter once only for all


    df_subsystem = df_data[subsystem_cols]
    # drop rows with all NaN values excluding datetime column
    # drop columns with all NaN values excluding datetime column
    df_subsystem = df_subsystem.dropna(axis=0, how="all", subset=df_subsystem.columns[1:])
    df_subsystem = df_subsystem.dropna(axis=1, how="all")

    # filter by component
    components = df_subsystem.columns.str.split(".").str[1].unique().tolist()

    component_list = st.multiselect("Select Components", ["ALL"] + components, default=["ALL"])
    if "ALL" not in component_list:
        component_cols = ["datetime"]
        for comp in component_list:
            component_cols += [col for col in df_subsystem.columns if comp in col ]
        df_subsystem = df_subsystem[component_cols]


    # filter by parameter
    parameters = df_subsystem.columns.str.split(".").str[2].unique().tolist()
    parameter_list = st.multiselect("Select Parameters", ["ALL"] + parameters, default=["ALL"])
    if "ALL" not in parameter_list:
        parameter_cols = ["datetime"]
        for param in parameter_list:
            parameter_cols += [col for col in df_subsystem.columns if param in col ]
        df_subsystem = df_subsystem[parameter_cols]


    df_subsystem = df_subsystem.dropna(axis=0, how="all", subset=df_subsystem.columns[1:])
    df_subsystem = df_subsystem.dropna(axis=1, how="all")

    if len(df_subsystem) > 0:

        st.write(subsystem, component_list, parameter_list)
        st.write(f"{len(df_subsystem)} rows, {len(df_subsystem.columns)-1} columns ({subsystem})")
        st.write(df_subsystem)

        fig = px.line(df_subsystem, x="datetime", y=df_subsystem.columns[1:], title=f"Health Monitor Data - {subsystem}")
        st.plotly_chart(fig, use_container_width=True)
    else:
        st.write("No data to display")


with tabs[2]:

    #### Beam Is On, Is Blanked ####

    fig_cols = st.columns(2)


    # select columns with beam on and beam blanked
    for filter_str in ["Beam Is On", "Is Blanked"]:

        cols = ["datetime"] + [col for col in df_data.columns if filter_str in col] # or "Beam Is On" in col 
        st.write(cols)
        df_beam = df_data[cols]

        df_beam = df_beam.dropna(axis=0, how="all", subset=df_beam.columns[1:])
        df_beam = df_beam.dropna(axis=1, how="all")

        # fillna with the previous row values
        df_beam = df_beam.fillna(method="ffill")
        # calculate duration
        df_beam["duration"] = df_beam["datetime"].diff().dt.total_seconds()
        df_beam = df_beam.dropna(axis=0, how="any", subset=df_beam.columns[1:])
        # st.dataframe(df_beam)


        # group by beam on and beam blanked
        fib_columns = [col for col in df_beam.columns if "FIB" in col]
        sem_columns = [col for col in df_beam.columns if "SEM" in col]
        # st.write(fib_columns, sem_columns)

        df_groupby_fib = df_beam.groupby(fib_columns).agg({"duration": "sum"}).reset_index()
        df_groupby_sem = df_beam.groupby(sem_columns).agg({"duration": "sum"}).reset_index()

        fig_cols[0].write(df_groupby_sem)
        fig_cols[1].write(df_groupby_fib)

        # convert duration to hours
        df_groupby_fib["duration"] = df_groupby_fib["duration"] / 3600
        df_groupby_sem["duration"] = df_groupby_sem["duration"] / 3600


        # plot as piecharts

        fig_fib = px.pie(df_groupby_fib, values="duration", title=f"FIB Beam {filter_str}", names=fib_columns[0], hover_data=df_groupby_fib.columns)
        fig_sem = px.pie(df_groupby_sem, values="duration", title=f"SEM Beam {filter_str}", names=sem_columns[0], hover_data=df_groupby_sem.columns)

        # show names on figure
        fig_fib.update_traces(textposition='inside', textinfo='percent+label')
        fig_sem.update_traces(textposition='inside', textinfo='percent+label')
        

        fig_cols[0].plotly_chart(fig_sem, use_container_width=True)
        fig_cols[1].plotly_chart(fig_fib, use_container_width=True)






# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemCryoDepositionWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(400, 300)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.label_title = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 2)
        self.label_sputter_stage_position = QtWidgets.QLabel(Dialog)
        self.label_sputter_stage_position.setObjectName("label_sputter_stage_position")
        self.gridLayout.addWidget(self.label_sputter_stage_position, 1, 0, 1, 1)
        self.comboBox_sputter_stage_position = QtWidgets.QComboBox(Dialog)
        self.comboBox_sputter_stage_position.setObjectName("comboBox_sputter_stage_position")
        self.gridLayout.addWidget(self.comboBox_sputter_stage_position, 1, 1, 1, 1)
        self.label_sputter_application_file = QtWidgets.QLabel(Dialog)
        self.label_sputter_application_file.setObjectName("label_sputter_application_file")
        self.gridLayout.addWidget(self.label_sputter_application_file, 2, 0, 1, 1)
        self.lineEdit_sputter_application_file = QtWidgets.QLineEdit(Dialog)
        self.lineEdit_sputter_application_file.setObjectName("lineEdit_sputter_application_file")
        self.gridLayout.addWidget(self.lineEdit_sputter_application_file, 2, 1, 1, 1)
        self.label_sputter_gis_position = QtWidgets.QLabel(Dialog)
        self.label_sputter_gis_position.setObjectName("label_sputter_gis_position")
        self.gridLayout.addWidget(self.label_sputter_gis_position, 3, 0, 1, 1)
        self.lineEdit_sputter_gis_position = QtWidgets.QLineEdit(Dialog)
        self.lineEdit_sputter_gis_position.setObjectName("lineEdit_sputter_gis_position")
        self.gridLayout.addWidget(self.lineEdit_sputter_gis_position, 3, 1, 1, 1)
        self.label_sputter_gas = QtWidgets.QLabel(Dialog)
        self.label_sputter_gas.setObjectName("label_sputter_gas")
        self.gridLayout.addWidget(self.label_sputter_gas, 4, 0, 1, 1)
        self.lineEdit_sputter_gas = QtWidgets.QLineEdit(Dialog)
        self.lineEdit_sputter_gas.setObjectName("lineEdit_sputter_gas")
        self.gridLayout.addWidget(self.lineEdit_sputter_gas, 4, 1, 1, 1)
        self.label_sputter_hfw = QtWidgets.QLabel(Dialog)
        self.label_sputter_hfw.setObjectName("label_sputter_hfw")
        self.gridLayout.addWidget(self.label_sputter_hfw, 5, 0, 1, 1)
        self.doubleSpinBox_sputter_hfw = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_hfw.setMaximum(1000.0)
        self.doubleSpinBox_sputter_hfw.setProperty("value", 30.0)
        self.doubleSpinBox_sputter_hfw.setObjectName("doubleSpinBox_sputter_hfw")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_hfw, 5, 1, 1, 1)
        self.label_sputter_length = QtWidgets.QLabel(Dialog)
        self.label_sputter_length.setObjectName("label_sputter_length")
        self.gridLayout.addWidget(self.label_sputter_length, 6, 0, 1, 1)
        self.doubleSpinBox_sputter_length = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_length.setMaximum(1000.0)
        self.doubleSpinBox_sputter_length.setProperty("value", 7.5)
        self.doubleSpinBox_sputter_length.setObjectName("doubleSpinBox_sputter_length")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_length, 6, 1, 1, 1)
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 7, 0, 1, 1)
        self.doubleSpinBox_sputter_beam_current = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_beam_current.setMaximum(1000.0)
        self.doubleSpinBox_sputter_beam_current.setProperty("value", 10.0)
        self.doubleSpinBox_sputter_beam_current.setObjectName("doubleSpinBox_sputter_beam_current")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_beam_current, 7, 1, 1, 1)
        self.label_sputter_time = QtWidgets.QLabel(Dialog)
        self.label_sputter_time.setObjectName("label_sputter_time")
        self.gridLayout.addWidget(self.label_sputter_time, 8, 0, 1, 1)
        self.doubleSpinBox_sputter_time = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_time.setMaximum(1000.0)
        self.doubleSpinBox_sputter_time.setProperty("value", 30.0)
        self.doubleSpinBox_sputter_time.setObjectName("doubleSpinBox_sputter_time")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_time, 8, 1, 1, 1)
        self.pushButton_run_sputter = QtWidgets.QPushButton(Dialog)
        self.pushButton_run_sputter.setObjectName("pushButton_run_sputter")
        self.gridLayout.addWidget(self.pushButton_run_sputter, 9, 0, 1, 2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label_title.setText(_translate("Dialog", "Cryo Sputter"))
        self.label_sputter_stage_position.setText(_translate("Dialog", "Stage Position"))
        self.label_sputter_application_file.setText(_translate("Dialog", "Application File"))
        self.lineEdit_sputter_application_file.setText(_translate("Dialog", "cryo_Pt_dep"))
        self.label_sputter_gis_position.setText(_translate("Dialog", "GIS Position"))
        self.lineEdit_sputter_gis_position.setText(_translate("Dialog", "cryo"))
        self.label_sputter_gas.setText(_translate("Dialog", "Gas"))
        self.lineEdit_sputter_gas.setText(_translate("Dialog", "Pt cryo"))
        self.label_sputter_hfw.setText(_translate("Dialog", "HFW (um)"))
        self.label_sputter_length.setText(_translate("Dialog", "Length (um)"))
        self.label.setText(_translate("Dialog", "Current (nA)"))
        self.label_sputter_time.setText(_translate("Dialog", "Time (s)"))
        self.pushButton_run_sputter.setText(_translate("Dialog", "Run Sputtering"))



from fibsem.microscope import FibsemMicroscope
import logging
import time
from fibsem.structures import BeamType


gis_protocol = {
    "application_file": "cryo_Pt_dep",
    "gas": "Pt cryo",
    "position": "cryo",
    "hfw": 3.0e-05 ,
    "length": 7.0e-06,
    "beam_current": 1.0e-8,
    "time": 30.0,
}

def deposit_platinum(
    microscope: FibsemMicroscope,
    protocol: dict = None,
    default_application_file: str = "Si",
):
    """Deposit platinum over the sample.

    Args:
        microscope (SdbMicroscopeClient): autoscript microscope instance
        protocol (dict): platinum protcol dictionary
        default_application_file (str): application file to use if none is specified
    Returns:
        None

    Raises:
        RuntimeError: Error Sputtering
    """

    if protocol is None:
        protocol = gis_protocol

    hfw = protocol["hfw"]
    line_pattern_length = protocol["length"]
    sputter_time = protocol["time"]
            

    # Setup
    microscope.setup_sputter(protocol=protocol)

    # Create sputtering pattern
    sputter_pattern = microscope.draw_sputter_pattern(hfw=hfw, line_pattern_length=line_pattern_length, sputter_time=sputter_time)

    # Run sputtering
    microscope.run_sputter(sputter_time=sputter_time, sputter_pattern=sputter_pattern)

    # Cleanup
    microscope.finish_sputter(application_file=default_application_file)


def cryo_deposition(microscope: FibsemMicroscope, protocol: dict = None, name: str = None):

    # get current position
    position = microscope.get_microscope_state().stage_position

    # move to deposition position
    if name is not None:
        
        # move to position
        from fibsem import utils
        deposition_position = utils._get_position(name)
        
        if deposition_position is None:
            raise RuntimeError(f"Position {name} requested but not found")
        
        logging.info(f"Moving to depositon position: {name}")
        microscope.safe_absolute_stage_movement(deposition_position)


    # move down
    from fibsem.structures import FibsemStagePosition
    microscope.move_stage_relative(FibsemStagePosition(z=-1e-3))

    # sputter
    deposit_platinum(microscope, protocol)

    # return to previous position
    microscope.safe_absolute_stage_movement(position)




import logging
from copy import deepcopy
import napari
import napari.utils.notifications
from PyQt5 import QtWidgets

import numpy as np
from fibsem import alignment
from fibsem.microscope import FibsemMicroscope, ThermoMicroscope, DemoMicroscope, TescanMicroscope
from fibsem.structures import (BeamType, MicroscopeSettings)
from fibsem.ui.qtdesigner_files import CurrentAlignmentWidget
from fibsem.ui import utils as ui_utils 
from fibsem.ui import _stylesheets



_UNSCALED_VALUES  = ["rotation", "size_ratio", "scan_direction", "cleaning_cross_section", "number"]
_ANGLE_KEYS = ["rotation"]
def _scale_value(key, value, scale):
    if key not in _UNSCALED_VALUES:
        return value * scale    
    return value


class FibsemAlignmentWidget(CurrentAlignmentWidget.Ui_BeamAlignment, QtWidgets.QWidget):
    # milling_param_changed = QtCore.pyqtSignal()

    def __init__(
        self,
        microscope: FibsemMicroscope = None,
        settings: MicroscopeSettings = None,
        viewer: napari.Viewer = None,
        parent=None,
    ):
        super(FibsemAlignmentWidget, self).__init__(parent=parent)
        self.setupUi(self)
        self.microscope = microscope
        self.settings = settings
        self.viewer = viewer
        self.parent = parent

        if isinstance(self.microscope, ThermoMicroscope) or isinstance(self.microscope, DemoMicroscope):
            currents = self.microscope.get_available_values("current", BeamType.ION)
            string_list = [str(num) for num in currents]
            self.comboBox_aligned_current.addItems(string_list)
            self.comboBox_ref_current.addItems(string_list)
        elif isinstance(self.microscope, TescanMicroscope):
            presets = self.microscope.get_available_values("presets", BeamType.ION)
            self.comboBox_aligned_current.addItems(presets)
            self.comboBox_ref_current.addItems(presets)
            self.label_2.setText("Reference Preset")
            self.label_3.setText("Aligned Preset")

        self.setup_connections()
        self.ref_layer = None
        self.aligned_layer = None   

    def setup_connections(self):
        self.checkBox_overlay.stateChanged.connect(self.update_overlay)
        self.pushButton_align_beam.clicked.connect(self.align_beam)
        self.pushButton_take_images.clicked.connect(self.take_images)
        self.pushButton_take_images.setStyleSheet(_stylesheets._GRAY_PUSHBUTTON_STYLE)
        self.pushButton_align_beam.setEnabled(False)
        self.pushButton_align_beam.setStyleSheet(_stylesheets._DISABLED_PUSHBUTTON_STYLE)

    def update_overlay(self):
        if self.checkBox_overlay.isChecked():
            self.aligned_layer.translate = [0.0, 0.0] 
            self.aligned_layer.opacity = 0.5
        else:
            self.aligned_layer.opacity = 1.0
            self.aligned_layer.translate = [0.0, self.ref_layer.data.shape[1] if self.ref_layer
                else self.aligned_layer.data.shape[1]]

    def align_beam(self):
        self.pushButton_align_beam.setEnabled(False)
        self.pushButton_align_beam.setText("Aligning...")
        self.pushButton_align_beam.setStyleSheet("color: orange")
        if isinstance(self.microscope, ThermoMicroscope) or isinstance(self.microscope, DemoMicroscope):
            self.microscope.set("current", float(self.comboBox_aligned_current.currentText()), beam_type=BeamType.ION)
        elif isinstance(self.microscope, TescanMicroscope):
            self.microscope.set("preset", self.comboBox_aligned_current.currentText(), beam_type=BeamType.ION)
        alignment.beam_shift_alignment(
            microscope=self.microscope,
            image_settings=self.settings.image,
            ref_image=self.ref_image,
            reduced_area=None,
        )
        shift = self.microscope.get("shift", BeamType.ION)
        ui_utils.message_box_ui(
            title="Beam Shift Alignment Complete",
            text=f"Beam Shifted by {shift.x}m, {shift.y}m",
            buttons=QtWidgets.QMessageBox.Ok,
        )
        string = self.listdone.toPlainText()
        string += (f"Aligned reference {self.comboBox_ref_current.currentText()} with {self.comboBox_aligned_current.currentText()}. Beam Shifted by {shift.x}m, {shift.y}m \n")
        self.listdone.setPlainText(string)
        self.take_images()
        self.parent.ref_current = self.comboBox_ref_current.currentText()
        self.pushButton_align_beam.setEnabled(True)
        self.pushButton_align_beam.setText("Align Beam")
        self.pushButton_align_beam.setStyleSheet(_stylesheets._GREEN_PUSHBUTTON_STYLE)

    def take_images(self):
        if isinstance(self.microscope, ThermoMicroscope) or isinstance(self.microscope, DemoMicroscope):
            self.microscope.set("current", float(self.comboBox_ref_current.currentText()), beam_type=BeamType.ION)
        elif isinstance(self.microscope, TescanMicroscope):
            self.microscope.set("preset", self.comboBox_ref_current.currentText(), beam_type=BeamType.ION)        
        self.settings.image.beam_type = BeamType.ION
        self.ref_image = self.microscope.acquire_image(self.settings.image)
        self.update_viewer(self.ref_image.data, "Reference")
        if isinstance(self.microscope, ThermoMicroscope) or isinstance(self.microscope, DemoMicroscope):
            self.microscope.set("current", float(self.comboBox_aligned_current.currentText()), beam_type=BeamType.ION)
        elif isinstance(self.microscope, TescanMicroscope):
            self.microscope.set("preset", self.comboBox_aligned_current.currentText(), beam_type=BeamType.ION)
        self.settings.image.beam_type = BeamType.ION
        self.aligned_image = self.microscope.acquire_image(self.settings.image)
        self.update_viewer(self.aligned_image.data, "Aligned")
        self.pushButton_align_beam.setEnabled(True)
        self.pushButton_align_beam.setStyleSheet(_stylesheets._GREEN_PUSHBUTTON_STYLE)


    def update_viewer(self, arr: np.ndarray, name: str = None):
        # arr = ui_utils._draw_crosshair(arr)

        try:
            self.viewer.layers[name].data = arr
        except:    
            layer = self.viewer.add_image(arr, name = name, colormap= "red" if name == "Reference" else "green")
        

        layer = self.viewer.layers[name]
        if self.ref_layer is None and name == "Reference":
            self.ref_layer = layer
        if self.aligned_layer is None and name == "Aligned":
            self.aligned_layer = layer
        
        # centre the camera
        if self.ref_layer:
            self.viewer.camera.center = [
                0.0,
                self.ref_layer.data.shape[0] / 2,
                self.ref_layer.data.shape[1],
            ]
            self.viewer.camera.zoom = 0.45

        if self.aligned_layer:
            translation = (
                self.viewer.layers["Reference"].data.shape[1]
                if self.ref_layer
                else arr.shape[1]
            )
            self.aligned_layer.translate = [0.0, translation]       

        if self.ref_layer:
            points = np.array([[-20, 200], [-20, self.ref_layer.data.shape[1] + 150]])
            string = ["REFERENCE", "ALIGNED"]
            text = {
                "string": string,
                "color": "white"
            }

            try:
                self.viewer.layers['label'].data = points
            except:    
                self.viewer.add_points(
                points,
                name="label",
                text=text,
                size=20,
                edge_width=7,
                edge_width_is_relative=False,
                edge_color='transparent',
                face_color='transparent',
                )   

def main():
    pass


if __name__ == "__main__":
    main()


import os
from pprint import pprint

import numpy as np
from autoscript_sdb_microscope_client.structures import (
    BitmapPatternDefinition, StagePosition)
from fibsem import acquire, milling, movement, utils
# from fibsem.ui import windows
from fibsem.structures import BeamType
from PIL import Image

BASE_PATH = os.path.dirname(__file__)

def save_profile_to_bmp(arr: np.ndarray, fname: str = "profile.bmp"):

    # scale values to int
    arr = (arr / np.max(arr)) * 255
    arr = arr.astype(np.uint8)

    # save profile
    Image.fromarray(arr).convert("RGB").save(fname)


def main():

    PROTOCOL_PATH = os.path.join(BASE_PATH, "protocol_lithography.yaml")
    microscope, settings = utils.setup_session(protocol_path=PROTOCOL_PATH)

    # lens plane
    microscope.system.electron.column_tilt = 0

    # move to the milling angle
    # stage_position = StagePosition(
    #     r=np.deg2rad(settings.protocol["stage_rotation"]),
    #     t=np.deg2rad(settings.protocol["stage_tilt"]),
    # )
    # movement.safe_absolute_stage_movement(microscope, stage_position)

    microscope.move_flat_to_beam(BeamType.ION)

    # eucentric, select position
    # windows.ask_user_movement(
    #     microscope,
    #     settings,
    #     msg_type="eucentric",
    #     msg="Select a position to mill the pattern.",
    # )

    # lens profile files
    npy_path = os.path.join(BASE_PATH, settings.protocol["profile"])
    bmp_path = os.path.join(BASE_PATH, "profile.bmp")

    # load milling properties
    profile = np.load(npy_path)
    pixel_size = settings.protocol["pixelsize"]
    lens_height = settings.protocol["milling"]["height"]
    lens_width = settings.protocol["milling"]["width"]

    # save profile to bmp
    save_profile_to_bmp(profile, bmp_path)

    # load bmp pattern
    bitmap_pattern = BitmapPatternDefinition()
    bitmap_pattern.load(bmp_path)

    # milling setup
    milling.setup_milling(
        microscope, application_file=settings.protocol["application_file"]
    )

    # surface milling
    microscope.patterning.create_bitmap(
        center_x=0,
        centre_y=0,
        width=lens_width,
        height=lens_height,
        depth=settings.protocol["initial_depth"],
        bitmap_pattern_definition=bitmap_pattern,
    )

    # mill bitmap
    microscope.patterning.create_bitmap(
        center_x=0,
        centre_y=0,
        width=lens_width,
        height=lens_height,
        depth=settings.protocol["milling"]["milling_depth"], 
        bitmap_pattern_definition=bitmap_pattern,
    )
    milling.run_milling(microscope, settings.protocol["milling"]["milling_current"])
    milling.finish_milling(microscope)


if __name__ == "__main__":
    main()



import os

CLASS_CONFIG_PATH = os.path.join(os.path.dirname(__file__), "segmentation_config.yaml")

import yaml
with open(CLASS_CONFIG_PATH) as f:
    CLASS_CONFIG = yaml.load(f, Loader=yaml.FullLoader)
    
CLASS_COLORS = CLASS_CONFIG["CLASS_COLORS"]
CLASS_LABELS = CLASS_CONFIG["CLASS_LABELS"]

import matplotlib.colors as mcolors
def convert_color_names_to_rgb(color_names: list[str]):
    if isinstance(color_names, dict):
        color_names = color_names.values()
    rgb_colors = [mcolors.to_rgb(color) for color in color_names]
    # Convert to 0-255 scale
    rgb_colors = [(int(r*255), int(g*255), int(b*255)) for r, g, b in rgb_colors]
    return rgb_colors

# map color names to rgb values
CLASS_COLORS_RGB = convert_color_names_to_rgb(CLASS_COLORS.values())


def get_colormap():
    return CLASS_COLORS_RGB

def get_class_labels():
    return CLASS_LABELS

CONFIG_PATH = os.path.join(os.path.dirname(__file__), "config.yml")


import napari

import logging

import fibsem
from fibsem import config as cfg
from fibsem.microscope import (DemoMicroscope, FibsemMicroscope,TescanMicroscope,
                               ThermoMicroscope)
from fibsem.structures import BeamType, MicroscopeSettings
from fibsem.ui.FibsemAlignmentWidget import FibsemAlignmentWidget
from fibsem.ui.FibsemImageViewer import FibsemImageViewer
from fibsem.ui.FibsemImageSettingsWidget import FibsemImageSettingsWidget
from fibsem.ui.FibsemManipulatorWidget import FibsemManipulatorWidget
from fibsem.ui.FibsemMillingWidget import FibsemMillingWidget
from fibsem.ui.FibsemMinimapWidget import FibsemMinimapWidget
from fibsem.ui.FibsemMovementWidget import FibsemMovementWidget
from fibsem.ui.FibsemSystemSetupWidget import FibsemSystemSetupWidget
from fibsem.ui.qtdesigner_files import FibsemUI
from fibsem.ui.utils import message_box_ui

from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import pyqtSignal


# TODO: add calibrate manipulator procedure for thermo here

class FibsemUI(FibsemUI.Ui_MainWindow, QtWidgets.QMainWindow):
    _minimap_signal = pyqtSignal(object)

    def __init__(self, viewer: napari.Viewer):
        super(FibsemUI, self).__init__()
        self.setupUi(self)

        self.label_title.setText(f"OpenFIBSEM v{fibsem.__version__}")

        self.viewer = viewer
        self.viewer.window._qt_viewer.dockLayerList.setVisible(False)
        self.viewer.window._qt_viewer.dockLayerControls.setVisible(False)

        self.microscope: FibsemMicroscope = None
        self.settings: MicroscopeSettings = None

        self.image_widget: FibsemImageSettingsWidget = None
        self.movement_widget: FibsemMovementWidget = None
        self.milling_widget: FibsemMillingWidget = None
        self.alignment_widget: FibsemAlignmentWidget = None
        self.manipulator_widget: FibsemManipulatorWidget = None
        self.image_viewer: FibsemImageViewer = None

        self.minimap_widget: FibsemMinimapWidget = None

        self.system_widget = FibsemSystemSetupWidget(
                microscope=self.microscope,
                settings=self.settings,
                viewer=self.viewer,
                parent=self,
            )
        
        self.tabWidget.addTab(self.system_widget, "Connection")
        self.setup_connections()
        self.ref_current = None
        self.update_ui()


    def setup_connections(self):
        from fibsem import utils
        self.system_widget.connected_signal.connect(self.connect_to_microscope)
        self.system_widget.disconnected_signal.connect(self.disconnect_from_microscope)
        # self.actionCurrent_alignment.triggered.connect(self.align_currents)
        if self.manipulator_widget is not None:
            self.actionManipulator_Positions_Calibration.triggered.connect(self.manipulator_widget.calibrate_manipulator_positions)
        self.actionOpen_Minimap.triggered.connect(self._open_minimap)

    def minimap_connection(self,positions=None):

        if self.minimap_widget is None:
            return
        else:
            self._minimap_signal.emit(positions)

    def _open_minimap(self):
        if self.microscope is None:
            napari.utils.notifications.show_warning(f"Please connect to a microscope first... [No Microscope Connected]")
            return

        if self.movement_widget is None:
            napari.utils.notifications.show_warning(f"Please connect to a microscope first... [No Movement Widget]")
            return

        # TODO: need to register this with the main ui somehow
        self.viewer2 = napari.Viewer(ndisplay=2)
        self.minimap_widget = FibsemMinimapWidget(self.microscope, self.settings, viewer=self.viewer2, parent=self)
        self.viewer2.window.add_dock_widget(
            self.minimap_widget, area="right", add_vertical_stretch=False, name="OpenFIBSEM Minimap"
        )
        self.minimap_widget._stage_position_moved.connect(self.movement_widget._stage_position_moved)

        self.minimap_widget._minimap_positions.connect(self.movement_widget.minimap_window_positions)

        self.minimap_widget.positions = self.movement_widget.positions
        self.minimap_widget._update_position_info()
        self.minimap_widget._update_viewer()

        napari.run(max_loop_level=2)



    def align_currents(self):
        second_viewer = napari.Viewer()
        self.alignment_widget = FibsemAlignmentWidget(settings=self.settings, microscope=self.microscope, viewer=second_viewer, parent = self)
        second_viewer.window.add_dock_widget(self.alignment_widget, name='Beam Alignment', area='right')
        self.alignment_widget.destroyed.connect(self.reset_currents)

    def reset_currents(self):
        if isinstance(self.microscope, ThermoMicroscope)or isinstance(self.microscope, DemoMicroscope):
            self.microscope.set("current", float(self.ref_current), BeamType.ION)
        if isinstance(self.microscope, TescanMicroscope):
            self.microscope.set("preset", self.ref_current, BeamType.ION)


    def update_ui(self):

        _microscope_connected = bool(self.microscope is not None)
        self.tabWidget.setTabVisible(1, _microscope_connected)
        self.tabWidget.setTabVisible(2, _microscope_connected)
        self.tabWidget.setTabVisible(3, _microscope_connected)
        self.tabWidget.setTabVisible(4, _microscope_connected)
        self.actionOpen_Minimap.setVisible(_microscope_connected)
        # self.actionCurrent_alignment.setVisible(_microscope_connected)
        self.actionManipulator_Positions_Calibration.setVisible(_microscope_connected)


    def connect_to_microscope(self):
        self.microscope = self.system_widget.microscope
        self.settings = self.system_widget.settings
        self.update_microscope_ui()
        self.update_ui()

    def disconnect_from_microscope(self):
        
        self.microscope = None
        self.settings = None
        self.update_microscope_ui()
        self.update_ui()
        self.image_widget = None
        self.movement_widget = None
        self.milling_widget = None

    def update_microscope_ui(self):

        if self.microscope is not None:
            # reusable components
            self.image_widget = FibsemImageSettingsWidget(
                microscope=self.microscope,
                image_settings=self.settings.image,
                viewer=self.viewer,
                parent=self,
            )
            self.movement_widget = FibsemMovementWidget(
                microscope=self.microscope,
                settings=self.settings,
                viewer=self.viewer,
                image_widget=self.image_widget,
                parent=self,
            )
            self.milling_widget = FibsemMillingWidget(
                microscope=self.microscope,
                settings=self.settings,
                viewer=self.viewer,
                image_widget=self.image_widget,
                parent=self,
            )
            if self.microscope.system.manipulator.enabled:
                self.manipulator_widget = FibsemManipulatorWidget(
                    microscope=self.microscope,
                    settings=self.settings,
                    viewer=self.viewer,
                    image_widget=self.image_widget,
                    parent=self,
                )
            else:
                self.manipulator_widget = None
  


            # add widgets to tabs
            self.tabWidget.addTab(self.image_widget, "Image")
            self.tabWidget.addTab(self.movement_widget, "Movement")
            self.tabWidget.addTab(self.milling_widget, "Milling")

            if self.microscope.system.manipulator.enabled:
                self.tabWidget.addTab(self.manipulator_widget, "Manipulator")

            self.system_widget.image_widget = self.image_widget
            self.system_widget.milling_widget = self.milling_widget
        
            # connect movement widget signal
            self.movement_widget.positions_signal.connect(self.minimap_connection)
            self.movement_widget.move_signal.connect(self.minimap_connection)

        else:
            if self.image_widget is None:
                return
            
            # remove tabs
            self.tabWidget.removeTab(4)
            self.tabWidget.removeTab(3)
            self.tabWidget.removeTab(2)
            self.tabWidget.removeTab(1)
            self.image_widget.clear_viewer()
            self.image_widget.deleteLater()
            self.movement_widget.deleteLater()
            self.milling_widget.deleteLater()
            if self.manipulator_widget is not None:
                self.manipulator_widget.deleteLater() 



def main():

    viewer = napari.Viewer(ndisplay=2)
    fibsem_ui = FibsemUI(viewer=viewer)
    viewer.window.add_dock_widget(fibsem_ui, 
                                  area="right", 
                                  add_vertical_stretch=True, 
                                  name=f"OpenFIBSEM v{fibsem.__version__}")
    napari.run()


if __name__ == "__main__":
    main()






import logging
from dataclasses import dataclass
from typing import Union

import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import numpy as np
from PIL import Image

from fibsem import constants, conversions
from fibsem.structures import Point, FibsemImage, FibsemRectangle
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure
from matplotlib.patches import Rectangle
from PyQt5.QtWidgets import QMessageBox, QSizePolicy, QVBoxLayout, QWidget
from fibsem.patterning import FibsemMillingStage
from fibsem.structures import (FibsemPatternSettings, 
                               FibsemRectangleSettings,  FibsemLineSettings, 
                               FibsemCircleSettings, FibsemBitmapSettings)
from fibsem.microscope import FibsemMicroscope
import napari
from fibsem.utils import load_yaml, save_yaml
import fibsem.patterning as patterning

# # TODO: clean up and refactor these (_WidgetPlot and _PlotCanvas)
# class _WidgetPlot(QWidget):
#     def __init__(self, *args, display_image, **kwargs):
#         QWidget.__init__(self, *args, **kwargs)
#         self.setLayout(QVBoxLayout())
#         self.canvas = _PlotCanvas(self, image=display_image)
#         self.layout().addWidget(self.canvas)


# class _PlotCanvas(FigureCanvasQTAgg):
#     def __init__(self, parent=None, image=None):
#         self.fig = Figure()
#         FigureCanvasQTAgg.__init__(self, self.fig)

#         self.setParent(parent)
#         FigureCanvasQTAgg.setSizePolicy(
#             self, QSizePolicy.Expanding, QSizePolicy.Expanding
#         )
#         FigureCanvasQTAgg.updateGeometry(self)
#         self.image = image
#         self.plot()
#         self.createConn()

#         self.figureActive = False
#         self.axesActive = None
#         self.cursorGUI = "arrow"
#         self.cursorChanged = False

#     def plot(self):
#         gs0 = self.fig.add_gridspec(1, 1)

#         self.ax11 = self.fig.add_subplot(gs0[0], xticks=[], yticks=[], title="")

#         if self.image.ndim == 3:
#             self.ax11.imshow(self.image,)
#         else:
#             self.ax11.imshow(self.image, cmap="gray")

#     def updateCanvas(self, event=None):
#         ax11_xlim = self.ax11.get_xlim()
#         ax11_xvis = ax11_xlim[1] - ax11_xlim[0]

#         while len(self.ax11.patches) > 0:
#             [p.remove() for p in self.ax11.patches]
#         while len(self.ax11.texts) > 0:
#             [t.remove() for t in self.ax11.texts]

#         ax11_units = ax11_xvis * 0.003
#         self.fig.canvas.draw()

#     def createConn(self):
#         self.fig.canvas.mpl_connect("figure_enter_event", self.activeFigure)
#         self.fig.canvas.mpl_connect("figure_leave_event", self.leftFigure)
#         self.fig.canvas.mpl_connect("button_press_event", self.mouseClicked)
#         self.ax11.callbacks.connect("xlim_changed", self.updateCanvas)

#     def activeFigure(self, event):
#         self.figureActive = True

#     def leftFigure(self, event):
#         self.figureActive = False
#         if self.cursorGUI != "arrow":
#             self.cursorGUI = "arrow"
#             self.cursorChanged = True

#     def mouseClicked(self, event):
#         x = event.xdata
#         y = event.ydata


# @dataclass
# class Crosshair:
#     rectangle_horizontal: plt.Rectangle
#     rectangle_vertical: plt.Rectangle


# # TODO update with Point
# def draw_crosshair(image, canvas, x: float = None, y: float = None, colour: str ="yellow"):
#     # draw crosshairs
#     crosshair = create_crosshair(image, x, y, colour=colour)
#     for patch in crosshair.__dataclass_fields__:
#         canvas.ax11.add_patch(getattr(crosshair, patch))
#         getattr(crosshair, patch).set_visible(True)

# # draw arrow
# def draw_arrow(p1: Point, p2: Point, canvas) -> None:
#     """Draw an arrow patch from p1 to p2"""
#     x1, y1 = p1.x, p1.y
#     x2, y2 = p2.x, p2.y
#     line = mpatches.Arrow(x1, y1, x2 - x1, y2 - y1, color="white")

#     # draw line
#     canvas.ax11.add_patch(line)
#     return


# def display_error_message(message, title="Error"):
#     """PyQt dialog box displaying an error message."""
#     logging.exception(message)

#     error_dialog = QMessageBox()
#     error_dialog.setIcon(QMessageBox.Critical)
#     error_dialog.setText(message)
#     error_dialog.setWindowTitle(title)
#     error_dialog.exec_()

#     return error_dialog


# def message_box_ui(title: str, text: str, buttons = QMessageBox.Yes | QMessageBox.No):

#     msg = QMessageBox()
#     msg.setWindowTitle(title)
#     msg.setText(text)
#     msg.setStandardButtons(buttons)
#     msg.exec_()

#     response = True if (msg.clickedButton() == msg.button(QMessageBox.Yes)) or (msg.clickedButton() == msg.button(QMessageBox.Ok) ) else False

#     return response


from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtWidgets import QGridLayout, QLabel

COLOURS = ["yellow", "cyan", "magenta", "lime", "orange", "hotpink", "green", "blue", "red", "purple"]


def set_arr_as_qlabel(
    arr: np.ndarray,
    label: QLabel,
    shape: tuple = (1536 // 4, 1024 // 4),
) -> QLabel:
    image = QImage(
        arr.data,
        arr.shape[1],
        arr.shape[0],
        QImage.Format_Grayscale8,
    )
    label.setPixmap(QPixmap.fromImage(image).scaled(*shape))

    return label


# def set_arr_as_qlabel_8(
#     arr: np.ndarray,
#     label: QLabel,
#     shape: tuple = (1536//4, 1024//4),
# ) -> QLabel:

#     image = QImage(
#         arr.data,
#         arr.shape[1],
#         arr.shape[0],
#         QImage.Format_Grayscale8,
#     )
#     label.setPixmap(QPixmap.fromImage(image).scaled(*shape))

#     return label


def convert_pattern_to_napari_circle(
    pattern_settings: FibsemCircleSettings, image: FibsemImage
):
    
    if not isinstance(pattern_settings, FibsemCircleSettings):
        raise ValueError("Pattern is not a circle")
    
    # image centre
    icy, icx = image.data.shape[0] // 2, image.data.shape[1] // 2
    # pixel size
    pixelsize_x, pixelsize_y = image.metadata.pixel_size.x, image.metadata.pixel_size.y


    # pattern to pixel coords
    r = int(pattern_settings.radius / pixelsize_x)
    cx = int(icx + (pattern_settings.centre_x / pixelsize_y))
    cy = int(icy - (pattern_settings.centre_y / pixelsize_y))

    # create corner coords
    xmin, ymin = cx - r, cy - r
    xmax, ymax = cx + r, cy + r

    # create circle
    shape = [[ymin, xmin], [ymin, xmax], [ymax, xmax], [ymax, xmin]]  # ??
    return shape


def convert_pattern_to_napari_line(
    pattern_settings: FibsemLineSettings, image: FibsemImage
) -> np.ndarray:
    # image centre
    icy, icx = image.data.shape[0] // 2, image.data.shape[1] // 2
    # pixel size
    pixelsize_x, pixelsize_y = image.metadata.pixel_size.x, image.metadata.pixel_size.y
    
    # extract pattern information from settings
    if not isinstance(pattern_settings, FibsemLineSettings):
        raise ValueError("Pattern is not a line")

    start_x = pattern_settings.start_x
    start_y = pattern_settings.start_y
    end_x = pattern_settings.end_x
    end_y = pattern_settings.end_y

    # pattern to pixel coords
    px0 = int(icx + (start_x / pixelsize_x))
    py0 = int(icy - (start_y / pixelsize_y))
    px1 = int(icx + (end_x / pixelsize_x))
    py1 = int(icy - (end_y / pixelsize_y))

    # napari shape format [[y_start, x_start], [y_end, x_end]])
    shape = [[py0, px0], [py1, px1]]
    return shape

def convert_pattern_to_napari_rect(
    pattern_settings: FibsemRectangleSettings, image: FibsemImage
) -> np.ndarray:
    # image centre
    icy, icx = image.data.shape[0] // 2, image.data.shape[1] // 2
    # pixel size
    pixelsize_x, pixelsize_y = image.metadata.pixel_size.x, image.metadata.pixel_size.y
    # extract pattern information from settings
    if isinstance(pattern_settings, FibsemLineSettings):
        pattern_width = pattern_settings.end_x - pattern_settings.start_x
        pattern_height = max(pattern_settings.end_y - pattern_settings.start_y, 0.5e-6)
        pattern_rotation = np.arctan2(
            pattern_height, pattern_width
        )  # TODO: line rotation doesnt work correctly, fix
        pattern_centre_x = (pattern_settings.end_x + pattern_settings.start_x) / 2
        pattern_centre_y = (pattern_settings.end_y + pattern_settings.start_y) / 2

    
    elif isinstance(pattern_settings, FibsemCircleSettings): #only used for out of bounds check
        pattern_width = 2*pattern_settings.radius
        pattern_height = 2*pattern_settings.radius
        pattern_centre_x = pattern_settings.centre_x
        pattern_centre_y = pattern_settings.centre_y
        pattern_rotation = 0

    else:
        pattern_width = pattern_settings.width
        pattern_height = pattern_settings.height
        pattern_centre_x = pattern_settings.centre_x
        pattern_centre_y = pattern_settings.centre_y
        pattern_rotation = pattern_settings.rotation
    # pattern to pixel coords
    w = int(pattern_width / pixelsize_x)
    h = int(pattern_height / pixelsize_y)
    cx = int(icx + (pattern_centre_x / pixelsize_y))
    cy = int(icy - (pattern_centre_y / pixelsize_y))
    r = -pattern_rotation  #
    xmin, xmax = -w / 2, w / 2
    ymin, ymax = -h / 2, h / 2
    px0 = cx + (xmin * np.cos(r) - ymin * np.sin(r))
    py0 = cy + (xmin * np.sin(r) + ymin * np.cos(r))
    px1 = cx + (xmax * np.cos(r) - ymin * np.sin(r))
    py1 = cy + (xmax * np.sin(r) + ymin * np.cos(r))
    px2 = cx + (xmax * np.cos(r) - ymax * np.sin(r))
    py2 = cy + (xmax * np.sin(r) + ymax * np.cos(r))
    px3 = cx + (xmin * np.cos(r) - ymax * np.sin(r))
    py3 = cy + (xmin * np.sin(r) + ymax * np.cos(r))
    # napari shape format
    shape = [[py0, px0], [py1, px1], [py2, px2], [py3, px3]]
    return shape

def create_crosshair_shape(centre_point: Point, image: FibsemImage,eb_image: FibsemImage):

    icy, icx = image.data.shape[0] // 2, image.data.shape[1] // 2

    pixelsize_x, pixelsize_y = image.metadata.pixel_size.x, image.metadata.pixel_size.y

    pattern_centre_x = centre_point.x
    pattern_centre_y = centre_point.y

    cx = int(icx + (pattern_centre_x / pixelsize_y))
    cy = int(icy - (pattern_centre_y / pixelsize_y))

    r_angles = [0,np.deg2rad(90)] #
    w = 40
    h = 1
    crosshair_shapes = []

    for r in r_angles:
        xmin, xmax = -w / 2, w / 2
        ymin, ymax = -h / 2, h / 2
        px0 = cx + (xmin * np.cos(r) - ymin * np.sin(r))
        py0 = cy + (xmin * np.sin(r) + ymin * np.cos(r))
        px1 = cx + (xmax * np.cos(r) - ymin * np.sin(r))
        py1 = cy + (xmax * np.sin(r) + ymin * np.cos(r))
        px2 = cx + (xmax * np.cos(r) - ymax * np.sin(r))
        py2 = cy + (xmax * np.sin(r) + ymax * np.cos(r))
        px3 = cx + (xmin * np.cos(r) - ymax * np.sin(r))
        py3 = cy + (xmin * np.sin(r) + ymax * np.cos(r))
        # napari shape format
        shape = [[py0, px0], [py1, px1], [py2, px2], [py3, px3]]
        if eb_image is not None:
                for c in shape:
                    c[1] += eb_image.data.shape[1]
        crosshair_shapes.append(shape)

    return crosshair_shapes






def convert_bitmap_pattern_to_napari_image(
        pattern_settings: FibsemBitmapSettings, image: FibsemImage
) -> np.ndarray:
    # image centre
    icy, icx = image.data.shape[0] // 2, image.data.shape[1] // 2
    # pixel size
    pixelsize_x, pixelsize_y = image.metadata.pixel_size.x, image.metadata.pixel_size.y

    resize_x = int(pattern_settings.width / pixelsize_x)
    resize_y = int(pattern_settings.height / pixelsize_y)

    
    image_bmp = Image.open(pattern_settings.path)
    image_resized = image_bmp.resize((resize_x, resize_y))
    image_rotated = image_resized.rotate(-pattern_settings.rotation, expand=True)
    img_array = np.array(image_rotated)

    pattern_centre_x = int(icx - pattern_settings.width/pixelsize_x/2) + image.data.shape[1] 
    pattern_centre_y = int(icy - pattern_settings.height/pixelsize_y/2)

    pattern_point_x = int(pattern_centre_x + pattern_settings.centre_x / pixelsize_x)
    pattern_point_y = int(pattern_centre_y - pattern_settings.centre_y / pixelsize_y)

    translate_position = (pattern_point_y,pattern_point_x)

    
    return img_array, translate_position

def convert_pattern_to_napari_image(pattern_settings: FibsemCircleSettings, image: FibsemImage) -> np.ndarray:
    """Convert a circle pattern to a napari image. Note: annulus can only be plotted as image"""
    # image centre
    icy, icx = image.data.shape[0] // 2, image.data.shape[1] // 2
    # pixel size
    pixelsize_x, pixelsize_y = image.metadata.pixel_size.x, image.metadata.pixel_size.y

    resize_x = int(2*pattern_settings.radius / pixelsize_x)
    resize_y = int(2*pattern_settings.radius / pixelsize_y)

    
    inner_radius_ratio = (pattern_settings.radius - pattern_settings.thickness)/pattern_settings.radius

    annulus_shape = _create_annulus_shape(width=resize_x, height=resize_y, inner_radius=inner_radius_ratio, outer_radius=1)
    # annulus_image = np.array(image_rotated)

    pattern_centre_x = int(icx - pattern_settings.radius/pixelsize_x) + image.data.shape[1] 
    pattern_centre_y = int(icy - pattern_settings.radius/pixelsize_y)

    pattern_point_x = int(pattern_centre_x + pattern_settings.centre_x / pixelsize_x)
    pattern_point_y = int(pattern_centre_y - pattern_settings.centre_y / pixelsize_y)

    translate_position = (pattern_point_y,pattern_point_x)

    return annulus_shape, translate_position
   
def _create_annulus_shape(width, height, inner_radius, outer_radius):
    # Create a grid of coordinates
    x = np.linspace(-1, 1, width)
    y = np.linspace(-1, 1, height)
    X, Y = np.meshgrid(x, y)
    distance = np.sqrt(X**2 + Y**2)
    # Generate the donut shape
    donut = np.logical_and(distance < outer_radius, distance > inner_radius).astype(int)
    return donut


def _remove_all_layers(viewer: napari.Viewer, layer_type = napari.layers.shapes.shapes.Shapes, _ignore: list[str] = []):

    # remove all shapes layers
    layers_to_remove = []
    layers_to_ignore = ["ruler_line","crosshair","scalebar","scalebar_value", "label"] + _ignore
    for layer in viewer.layers:

        if layer.name in layers_to_ignore:
            continue
        if isinstance(layer, layer_type) or layer.name in ["bmp_Image","annulus_Image"]:
            layers_to_remove.append(layer)
    for layer in layers_to_remove:
        viewer.layers.remove(layer)  # Not removing the second layer?


def _draw_patterns_in_napari(
    viewer: napari.Viewer,
    ib_image: FibsemImage,
    eb_image: FibsemImage,
    milling_stages: list[FibsemMillingStage],
    draw_crosshair: bool = True,
):

    # colour wheel
    COLOURS = ["yellow", "cyan", "magenta", "lime", "orange", "hotpink", "green", "blue", "red", "purple"]

    # convert fibsem patterns to napari shapes
    import time

    t_1 = time.time()
    _ignore = []
    for i, stage in enumerate(milling_stages):
        shape_patterns = []
        shape_types = []
        t0 = time.time()

        patterns = stage.pattern.patterns
        point = stage.pattern.point
        name = stage.name
        is_line_pattern = False

        for pattern_settings in patterns:
            if isinstance(pattern_settings, FibsemBitmapSettings):
                if pattern_settings.path == None or pattern_settings.path == '':
                    continue

                bmp_Image, translate_position = convert_bitmap_pattern_to_napari_image(pattern_settings=pattern_settings, image=ib_image)
                if "bmp_Image" in viewer.layers:
                    viewer.layers.remove(viewer.layers["bmp_Image"])
                viewer.add_image(bmp_Image,translate=translate_position,name="bmp_Image")
                shape_patterns = []
                _ignore.append("bmp_Image")
                continue

            elif isinstance(pattern_settings, FibsemCircleSettings):
                if pattern_settings.thickness != 0:
                    annulus_image, translate_position = convert_pattern_to_napari_image(pattern_settings=pattern_settings, image=ib_image)
                    if "annulus_Image" in viewer.layers:
                        viewer.layers.remove(viewer.layers["annulus_Image"])
                    viewer.add_image(annulus_image,translate=translate_position,name="annulus_Image",blending="additive",colormap=COLOURS[i % len(COLOURS)],opacity=0.4)
                    shape_patterns = []
                    _ignore.append("annulus_Image")
                    continue
                else:
                    shape = convert_pattern_to_napari_circle(pattern_settings=pattern_settings, image=ib_image)
                    shape_types.append("ellipse")
                    _ignore.append(name)

            elif isinstance(pattern_settings, FibsemLineSettings):
                shape = convert_pattern_to_napari_line(pattern_settings=pattern_settings, image=ib_image)
                shape_types.append("line")
                _ignore.append(name)
                is_line_pattern = True
            
            else:
                shape = convert_pattern_to_napari_rect(
                    pattern_settings=pattern_settings, image=ib_image
                )
                shape_types.append("rectangle")
                _ignore.append(name)

            # offset the x coord by image width
            if eb_image is not None:
                for c in shape:
                    c[1] += eb_image.data.shape[1]
            shape_patterns.append(shape)
        
        t1 = time.time()

        if len(shape_patterns) > 0:
            
            if draw_crosshair:
                crosshair_shapes = create_crosshair_shape(centre_point=point, image=ib_image, eb_image=eb_image)
                crosshair_shape_types = ["rectangle","rectangle"]
                shape_patterns += crosshair_shapes
                shape_types += crosshair_shape_types

            # _name = f"Stage {i+1:02d}"
            if name in viewer.layers:
                viewer.layers[name].data = []
                viewer.layers[name].data = shape_patterns
                viewer.layers[name].shape_type = shape_types
                viewer.layers[name].edge_color = COLOURS[i % len(COLOURS)]
                viewer.layers[name].face_color=COLOURS[i % len(COLOURS)]
            else:
                viewer.add_shapes(
                    shape_patterns,
                    name=name,
                    shape_type=shape_types,
                    edge_width=0.5,
                    edge_color=COLOURS[i % len(COLOURS)],
                    face_color=COLOURS[i % len(COLOURS)],
                    opacity=0.5,
                    blending="translucent",
                )

            if is_line_pattern:
                viewer.layers[name].edge_width = 3

        t2 = time.time()
        # remove all un-updated layers (assume they have been deleted)        
        _remove_all_layers(viewer=viewer, layer_type=napari.layers.shapes.shapes.Shapes, _ignore=_ignore)#[stage.name for stage in milling_stages])
        t3 = time.time()
        logging.debug(f"_DRAW_SHAPES: CONVERT: {t1-t0}, ADD/UPDATE: {t2-t1}, REMOVE: {t3-t2}")
    t_2 = time.time()
    logging.debug(f"_DRAW_SHAPES: total time: {t_2-t_1}")

def message_box_ui(title: str, text: str, buttons=QMessageBox.Yes | QMessageBox.No):
    msg = QMessageBox()
    msg.setWindowTitle(title)
    msg.setText(text)
    msg.setStandardButtons(buttons)
    msg.exec_()

    response = (
        True
        if (msg.clickedButton() == msg.button(QMessageBox.Yes))
        or (msg.clickedButton() == msg.button(QMessageBox.Ok))
        else False
    )

    return response

from PyQt5 import QtGui
def _display_logo(path, label, shape=[50, 50]):
    label.setScaledContents(True)
    label.setFixedSize(*shape)
    label.setPixmap(QtGui.QPixmap(path))


def create_combobox_message_box(text: str, title: str, options: list, parent = None):
    # create a q message box with combobox
    msg = QtWidgets.QMessageBox(parent=parent)
    msg.setIcon(QtWidgets.QMessageBox.Information)
    msg.setText(text)
    msg.setWindowTitle(title)
    msg.setStandardButtons(QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel)

    # create a combobox
    combobox = QtWidgets.QComboBox(msg)
    combobox.addItems(options)

    # add combobox to message box
    msg.layout().addWidget(combobox, 1, 1)

    # show message box
    msg.exec_()

    # get the selected milling pattern

    if msg.result() == QtWidgets.QMessageBox.Ok:
        selected = combobox.currentText()

        return selected
    
    return None


        
def _draw_crosshair(viewer: napari.Viewer, eb_image, ib_image,is_checked=False, width: float = 0.15) -> None:


    layers_in_napari = []
    for layer in viewer.layers:
        layers_in_napari.append(layer.name)


    if is_checked:

        centre_points = [ [eb_image.data.shape[0]//2, eb_image.data.shape[1]//2],[ib_image.data.shape[0]//2, eb_image.data.shape[1] + ib_image.data.shape[1]//2]]

        crosshairs = []

        for i,point in enumerate(centre_points):
            
            eb_location_r,eb_location_c = point[0], point[1]
            crosshair_length = width*point[0]
            horizontal_line = [ [eb_location_r, eb_location_c - crosshair_length],[eb_location_r,eb_location_c + crosshair_length] ]
            vertical_line = [ [eb_location_r - crosshair_length, eb_location_c],[eb_location_r + crosshair_length,eb_location_c] ]

            crosshairs.append(horizontal_line)
            crosshairs.append(vertical_line)

        if f"crosshair" not in layers_in_napari:

            viewer.add_shapes(data=crosshairs, shape_type='line', edge_width=5, edge_color='yellow', face_color='yellow', opacity=0.8, blending='translucent', name=f'crosshair')
        else:
            viewer.layers[f"crosshair"].data = crosshairs
            viewer.layers[f"crosshair"].opacity = 0.8

    else:
        
        if f"crosshair" in layers_in_napari :
            viewer.layers["crosshair"].opacity = 0
           
    return 

def _scale_length_value(hfw: float) -> float:

    scale_length_value = hfw*constants.METRE_TO_MICRON*0.2

    if scale_length_value > 0 and scale_length_value < 100:
        scale_length_value = round(scale_length_value/5)*5
    if scale_length_value > 100 and scale_length_value < 500:
        scale_length_value = round(scale_length_value/25)*25
    if scale_length_value > 500 and scale_length_value < 1000:
        scale_length_value = round(scale_length_value/50)*50

    scale_ratio = scale_length_value/(hfw*constants.METRE_TO_MICRON)

    return scale_ratio,scale_length_value

def _draw_scalebar(viewer: napari.Viewer, eb_image, ib_image,is_checked=False, width: float = 0.1) -> np.ndarray:

    layers_in_napari = []
    for layer in viewer.layers:
        layers_in_napari.append(layer.name)


    if is_checked:

        location_points = [ [int(eb_image.data.shape[0]*0.9), int(eb_image.data.shape[1]*0.15)],[int(ib_image.data.shape[0]*0.9), int(eb_image.data.shape[1] + ib_image.data.shape[1]*0.15)]]

        if is_checked:
            
            # making the scale bar line
            scale_bar_shape = []

            for i,point in enumerate(location_points):
                
                

                if i == 0:
                    scale_ratio,eb_scale = _scale_length_value(eb_image.metadata.image_settings.hfw)
                    length = scale_ratio*eb_image.data.shape[1]
                else:
                    scale_ratio,ib_scale = _scale_length_value(ib_image.metadata.image_settings.hfw)
                    length = scale_ratio*ib_image.data.shape[1]

                main_line = [[point[0]+25, int(point[1]-0.5*length)], [point[0]+25, int(point[1]+0.5*length)]]
        
                left_line = [[point[0]+50, int(point[1]-0.5*length)], [point[0], int(point[1]-0.5*length)]]
                right_line = [[point[0]+50, int(point[1]+0.5*length)], [point[0], int(point[1]+0.5*length)]]

                scale_bar_shape.append(main_line)
                scale_bar_shape.append(left_line)
                scale_bar_shape.append(right_line)

            if "scalebar" not in layers_in_napari:
                
                viewer.add_shapes(
                    data=scale_bar_shape,
                    shape_type='line',
                    edge_width=5,
                    edge_color='yellow',
                    name='scalebar'
                )
            else:

                viewer.layers["scalebar"].data = scale_bar_shape
                viewer.layers["scalebar"].opacity = 1

            ## making the scale bar value

            

            text = {
                "string": [f"{eb_scale} um",f"{ib_scale} um"],
                "color":"white"
            }

            if "scalebar" not in layers_in_napari:

                viewer.add_points(
                    data=location_points,
                    text=text,
                    size=20,
                    name="scalebar_value",
                    edge_color='transparent',
                    face_color='transparent',

                )
            else:
                viewer.layers["scalebar_value"].data = location_points
                viewer.layers["scalebar_value"].text = text
                viewer.layers["scalebar_value"].opacity = 1

    else:

        if "scalebar" in layers_in_napari:
            viewer.layers["scalebar"].opacity = 0
            viewer.layers["scalebar_value"].opacity = 0


    


def convert_point_to_napari(resolution: list, pixel_size: float, centre: Point):
    icy, icx = resolution[1] // 2, resolution[0] // 2

    cx = int(icx + (centre.x / pixel_size))
    cy = int(icy - (centre.y / pixel_size))

    return Point(cx, cy)


def validate_pattern_placement(
    patterns: list[FibsemPatternSettings], resolution: list, shape: list[list[float]]
):
    x_lim = resolution[0]
    y_lim = resolution[1]

    for coordinate in shape:
        x_coord = coordinate[1]
        y_coord = coordinate[0]

        if x_coord < 0 or x_coord > x_lim:
            return False
        if y_coord < 0 or y_coord > y_lim:
            return False

    return True


from fibsem import config as cfg
from pathlib import Path


# TODO: add filters for file types


def _get_directory_ui(
    msg: str = "Select a directory", path: Path = cfg.LOG_PATH, parent=None
) -> Path:
    path = QtWidgets.QFileDialog.getExistingDirectory(parent, msg, directory=path)
    return path


def _get_file_ui(
    msg: str = "Select a file",
    path: Path = cfg.LOG_PATH,
    _filter: str = "*yaml",
    parent=None,
) -> Path:
    path, _ = QtWidgets.QFileDialog.getOpenFileName(
        parent, msg, directory=path, filter=_filter
    )
    return path


def _get_save_file_ui(
    msg: str = "Select a file",
    path: Path = cfg.LOG_PATH,
    _filter: str = "*yaml",
    parent=None,
) -> Path:
    path, _ = QtWidgets.QFileDialog.getSaveFileName(
        parent,
        msg,
        directory=path,
        filter=_filter,
    )
    return path


def _get_text_ui(
    msg: str = "Enter text",
    title: str = "Text Entry",
    default: str = "UserText",
    parent=None,
) -> tuple[str, bool]:
    text, okPressed = QtWidgets.QInputDialog.getText(
        parent,
        title,
        msg,
        QtWidgets.QLineEdit.Normal,
        default,
    )

    return text, okPressed

def _draw_milling_stages_on_image(image: FibsemImage, milling_stages: list[FibsemMillingStage], show: bool = True):

    viewer = napari.Viewer()
    viewer.add_image(image.data, name='test_image')
    _draw_patterns_in_napari(viewer=viewer,ib_image=image,eb_image=None,milling_stages=milling_stages)
    screenshot = viewer.screenshot()
    fig, ax = plt.subplots(figsize=(10, 10))
    ax.imshow(screenshot)
    viewer.close()

    for i,stage in enumerate(milling_stages):
    
        plt.plot(0,0,'-',color=COLOURS[i % len(COLOURS)],label=stage.name)

    ax.axis('off')
    ax.legend()
    if show:
        plt.show()
    
    return fig

def _calculate_fiducial_area_v2(image: FibsemImage, fiducial_centre: Point, fiducial_length:float)->tuple[FibsemRectangle, bool]:
    pixelsize = image.metadata.pixel_size.x
    
    fiducial_centre.y = -fiducial_centre.y
    fiducial_centre_px = conversions.convert_point_from_metres_to_pixel(
        fiducial_centre, pixelsize
    )

    rcx = fiducial_centre_px.x / image.metadata.image_settings.resolution[0] + 0.5
    rcy = fiducial_centre_px.y / image.metadata.image_settings.resolution[1] + 0.5

    fiducial_length_px = (
        conversions.convert_metres_to_pixels(fiducial_length, pixelsize) * 2 # SCALE_FACTOR
    )
    h_offset = fiducial_length_px / image.metadata.image_settings.resolution[0] / 2
    v_offset = fiducial_length_px / image.metadata.image_settings.resolution[1] / 2

    left = rcx - h_offset
    top = rcy - v_offset
    width = 2 * h_offset
    height = 2 * v_offset

    if left < 0 or (left + width) > 1 or top < 0 or (top + height) > 1:
        flag = True
    else:
        flag = False

    fiducial_area = FibsemRectangle(left, top, width, height)

    return fiducial_area, flag

    
def show_information_dialog(microscope: FibsemMicroscope, parent=None):
    import fibsem
    
    fibsem_version = fibsem.__version__
    autolamella_version = "Not Installed"
    try:
        import autolamella
        autolamella_version = autolamella.__version__
    except:
        pass
    
    info = microscope.system.info

    text = f"""
    OpenFIBSEM Information:
    OpenFIBSEM: {fibsem_version}
    AutoLamella: {autolamella_version}

    Microscope Information:
    Name: {info.name}
    Manufacturer: {info.manufacturer}
    Model: {info.model}
    Serial Number: {info.serial_number}
    Firmware Version: {info.hardware_version}
    Software Version: {info.software_version}
    """

    # create a qdialog box with information
    msg = QtWidgets.QMessageBox(parent=parent)
    msg.setIcon(QtWidgets.QMessageBox.Information)
    msg.setWindowTitle("Information")
    msg.setText(text)
    msg.setStandardButtons(QtWidgets.QMessageBox.Ok)

    # exec
    msg.exec_()



import os
import glob
import pandas as pd
from pathlib import Path
import argparse

def _split_dataset(DATA_PATH: Path, OUTPUT_PATH: Path):

    TRAIN_PATH = os.path.join(OUTPUT_PATH, "train" )
    TEST_PATH = os.path.join(OUTPUT_PATH, "test")

    # make dirs
    os.makedirs(os.path.join(TRAIN_PATH, "labels"), exist_ok=True)
    os.makedirs(os.path.join(TEST_PATH, "labels"), exist_ok=True)

    df = pd.read_csv(os.path.join(DATA_PATH, "data.csv"))
    df["path"] = df["image"].apply(lambda x: os.path.join(DATA_PATH, f"{x}.tif"))
    df["mask_path"] = df["image"].apply(lambda x: os.path.join(DATA_PATH, "mask", f"{x}.tif"))

    print(f"total: {len(df)} images")

    # corrected: did the user overwrite the model prediction
    # False: the model prediction was correct -> test set
    # True: the model prediction was incorrect -> train set

    # split the data into train and test
    df_train = df[df["corrected"] == True]
    df_test = df[df["corrected"] == False]

    print(f"train: {len(df_train)} images")
    print(f"test: {len(df_test)} images")

    response = input(f"Move data to {OUTPUT_PATH}? [y/n]")

    if response == "y":
        # move the images and masks to the correct folder
        for path, mask_path in zip(df_train["path"].unique(), df_train["mask_path"].unique()):
            os.rename(path, os.path.join(TRAIN_PATH, os.path.basename(path)))
            os.rename(mask_path, os.path.join(TRAIN_PATH, "labels", os.path.basename(mask_path)))

        for path, mask_path in zip(df_test["path"].unique(), df_test["mask_path"].unique()):
            os.rename(path, os.path.join(TEST_PATH, os.path.basename(path)))
            os.rename(mask_path, os.path.join(TEST_PATH, "labels", os.path.basename(mask_path)))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--data_path", type=str, dest="data_path", action="store", help="Path to the data folder")
    parser.add_argument("--output_path", type=str, dest="output_path", action="store", help="Path to the output folder")

    args = parser.parse_args()

    _split_dataset(args.data_path, args.output_path)


if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemDetectionWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(365, 646)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.checkBox_show_info = QtWidgets.QCheckBox(Form)
        self.checkBox_show_info.setObjectName("checkBox_show_info")
        self.gridLayout.addWidget(self.checkBox_show_info, 14, 2, 1, 1)
        self.comboBox_feature_2 = QtWidgets.QComboBox(Form)
        self.comboBox_feature_2.setObjectName("comboBox_feature_2")
        self.gridLayout.addWidget(self.comboBox_feature_2, 12, 2, 1, 1)
        self.label_num_classes = QtWidgets.QLabel(Form)
        self.label_num_classes.setObjectName("label_num_classes")
        self.gridLayout.addWidget(self.label_num_classes, 9, 0, 1, 1)
        self.lineEdit_checkpoint = QtWidgets.QLineEdit(Form)
        self.lineEdit_checkpoint.setObjectName("lineEdit_checkpoint")
        self.gridLayout.addWidget(self.lineEdit_checkpoint, 8, 1, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 31, 0, 1, 3)
        self.label_instructions = QtWidgets.QLabel(Form)
        self.label_instructions.setWordWrap(True)
        self.label_instructions.setObjectName("label_instructions")
        self.gridLayout.addWidget(self.label_instructions, 29, 0, 1, 3)
        self.label_image_path_num = QtWidgets.QLabel(Form)
        self.label_image_path_num.setAlignment(QtCore.Qt.AlignCenter)
        self.label_image_path_num.setObjectName("label_image_path_num")
        self.gridLayout.addWidget(self.label_image_path_num, 26, 1, 1, 1)
        self.pushButton_previous_image = QtWidgets.QPushButton(Form)
        self.pushButton_previous_image.setObjectName("pushButton_previous_image")
        self.gridLayout.addWidget(self.pushButton_previous_image, 26, 0, 1, 1)
        self.label_title = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 3)
        self.spinBox_num_classes = QtWidgets.QSpinBox(Form)
        self.spinBox_num_classes.setObjectName("spinBox_num_classes")
        self.gridLayout.addWidget(self.spinBox_num_classes, 9, 1, 1, 2)
        self.label_header_features = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_header_features.setFont(font)
        self.label_header_features.setObjectName("label_header_features")
        self.gridLayout.addWidget(self.label_header_features, 11, 0, 1, 3)
        self.pushButton_run_feature_detection = QtWidgets.QPushButton(Form)
        self.pushButton_run_feature_detection.setObjectName("pushButton_run_feature_detection")
        self.gridLayout.addWidget(self.pushButton_run_feature_detection, 15, 0, 1, 3)
        self.checkBox_use_segmentation = QtWidgets.QCheckBox(Form)
        self.checkBox_use_segmentation.setChecked(True)
        self.checkBox_use_segmentation.setObjectName("checkBox_use_segmentation")
        self.gridLayout.addWidget(self.checkBox_use_segmentation, 1, 0, 1, 1)
        self.pushButton_load_images = QtWidgets.QPushButton(Form)
        self.pushButton_load_images.setObjectName("pushButton_load_images")
        self.gridLayout.addWidget(self.pushButton_load_images, 25, 0, 1, 3)
        self.label_encoder = QtWidgets.QLabel(Form)
        self.label_encoder.setObjectName("label_encoder")
        self.gridLayout.addWidget(self.label_encoder, 7, 0, 1, 1)
        self.label_info = QtWidgets.QLabel(Form)
        self.label_info.setMinimumSize(QtCore.QSize(0, 0))
        self.label_info.setText("")
        self.label_info.setObjectName("label_info")
        self.gridLayout.addWidget(self.label_info, 16, 0, 1, 3)
        self.checkBox_use_feature_detection = QtWidgets.QCheckBox(Form)
        self.checkBox_use_feature_detection.setChecked(True)
        self.checkBox_use_feature_detection.setObjectName("checkBox_use_feature_detection")
        self.gridLayout.addWidget(self.checkBox_use_feature_detection, 1, 1, 1, 1)
        self.pushButton_continue = QtWidgets.QPushButton(Form)
        self.pushButton_continue.setObjectName("pushButton_continue")
        self.gridLayout.addWidget(self.pushButton_continue, 30, 0, 1, 3)
        self.label_checkpoint = QtWidgets.QLabel(Form)
        self.label_checkpoint.setObjectName("label_checkpoint")
        self.gridLayout.addWidget(self.label_checkpoint, 8, 0, 1, 1)
        self.lineEdit_encoder = QtWidgets.QLineEdit(Form)
        self.lineEdit_encoder.setObjectName("lineEdit_encoder")
        self.gridLayout.addWidget(self.lineEdit_encoder, 7, 1, 1, 2)
        self.label_dataset = QtWidgets.QLabel(Form)
        self.label_dataset.setObjectName("label_dataset")
        self.gridLayout.addWidget(self.label_dataset, 24, 0, 1, 1)
        self.pushButton_next_image = QtWidgets.QPushButton(Form)
        self.pushButton_next_image.setObjectName("pushButton_next_image")
        self.gridLayout.addWidget(self.pushButton_next_image, 26, 2, 1, 1)
        self.label_header_model = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_header_model.setFont(font)
        self.label_header_model.setObjectName("label_header_model")
        self.gridLayout.addWidget(self.label_header_model, 2, 0, 1, 3)
        self.label_feature = QtWidgets.QLabel(Form)
        self.label_feature.setObjectName("label_feature")
        self.gridLayout.addWidget(self.label_feature, 12, 0, 1, 1)
        self.checkBox_use_evaluation = QtWidgets.QCheckBox(Form)
        self.checkBox_use_evaluation.setObjectName("checkBox_use_evaluation")
        self.gridLayout.addWidget(self.checkBox_use_evaluation, 1, 2, 1, 1)
        self.pushButton_load_model = QtWidgets.QPushButton(Form)
        self.pushButton_load_model.setObjectName("pushButton_load_model")
        self.gridLayout.addWidget(self.pushButton_load_model, 10, 0, 1, 3)
        self.label_header_evalulation = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_header_evalulation.setFont(font)
        self.label_header_evalulation.setObjectName("label_header_evalulation")
        self.gridLayout.addWidget(self.label_header_evalulation, 22, 0, 1, 3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 27, 0, 1, 3)
        self.lineEdit_image_path_folder = QtWidgets.QLineEdit(Form)
        self.lineEdit_image_path_folder.setObjectName("lineEdit_image_path_folder")
        self.gridLayout.addWidget(self.lineEdit_image_path_folder, 24, 1, 1, 2)
        self.checkBox_move_features = QtWidgets.QCheckBox(Form)
        self.checkBox_move_features.setObjectName("checkBox_move_features")
        self.gridLayout.addWidget(self.checkBox_move_features, 14, 1, 1, 1)
        self.comboBox_feature_1 = QtWidgets.QComboBox(Form)
        self.comboBox_feature_1.setObjectName("comboBox_feature_1")
        self.gridLayout.addWidget(self.comboBox_feature_1, 12, 1, 1, 1)
        self.checkBox_all_features = QtWidgets.QCheckBox(Form)
        self.checkBox_all_features.setObjectName("checkBox_all_features")
        self.gridLayout.addWidget(self.checkBox_all_features, 14, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.checkBox_use_segmentation, self.checkBox_use_feature_detection)
        Form.setTabOrder(self.checkBox_use_feature_detection, self.checkBox_use_evaluation)
        Form.setTabOrder(self.checkBox_use_evaluation, self.lineEdit_encoder)
        Form.setTabOrder(self.lineEdit_encoder, self.lineEdit_checkpoint)
        Form.setTabOrder(self.lineEdit_checkpoint, self.spinBox_num_classes)
        Form.setTabOrder(self.spinBox_num_classes, self.pushButton_load_model)
        Form.setTabOrder(self.pushButton_load_model, self.comboBox_feature_1)
        Form.setTabOrder(self.comboBox_feature_1, self.comboBox_feature_2)
        Form.setTabOrder(self.comboBox_feature_2, self.checkBox_all_features)
        Form.setTabOrder(self.checkBox_all_features, self.checkBox_move_features)
        Form.setTabOrder(self.checkBox_move_features, self.checkBox_show_info)
        Form.setTabOrder(self.checkBox_show_info, self.pushButton_run_feature_detection)
        Form.setTabOrder(self.pushButton_run_feature_detection, self.lineEdit_image_path_folder)
        Form.setTabOrder(self.lineEdit_image_path_folder, self.pushButton_load_images)
        Form.setTabOrder(self.pushButton_load_images, self.pushButton_previous_image)
        Form.setTabOrder(self.pushButton_previous_image, self.pushButton_next_image)
        Form.setTabOrder(self.pushButton_next_image, self.pushButton_continue)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.checkBox_show_info.setText(_translate("Form", "Show Info"))
        self.label_num_classes.setText(_translate("Form", "No. Classes"))
        self.label_instructions.setText(_translate("Form", "Drag to move the features, when finished press continue."))
        self.label_image_path_num.setText(_translate("Form", "TextLabel"))
        self.pushButton_previous_image.setText(_translate("Form", "Previous"))
        self.label_title.setText(_translate("Form", "Feature Detection"))
        self.label_header_features.setText(_translate("Form", "Feature Detection"))
        self.pushButton_run_feature_detection.setText(_translate("Form", "Run Feature Detection"))
        self.checkBox_use_segmentation.setText(_translate("Form", "Segmentation"))
        self.pushButton_load_images.setText(_translate("Form", "Load Images"))
        self.label_encoder.setText(_translate("Form", "Encoder"))
        self.checkBox_use_feature_detection.setText(_translate("Form", "Feature Detection"))
        self.pushButton_continue.setText(_translate("Form", "Continue"))
        self.label_checkpoint.setText(_translate("Form", "Checkpoint"))
        self.label_dataset.setText(_translate("Form", "Dataset"))
        self.pushButton_next_image.setText(_translate("Form", "Next"))
        self.label_header_model.setText(_translate("Form", "Segmentation Model"))
        self.label_feature.setText(_translate("Form", "Features"))
        self.checkBox_use_evaluation.setText(_translate("Form", "Evalulation"))
        self.pushButton_load_model.setText(_translate("Form", "Load Model"))
        self.label_header_evalulation.setText(_translate("Form", "Evaluation Mode"))
        self.checkBox_move_features.setText(_translate("Form", "Move Features"))
        self.checkBox_all_features.setText(_translate("Form", "All Features"))



from fibsem import utils
from fibsem.structures import FibsemMillingSettings, FibsemRectangleSettings, FibsemLineSettings
from fibsem import milling
import logging

"""
This script demonstrates how to use the milling module to mill a rectangle and two lines.

The script will:
    - connect to the microscope
    - setup milling
    - draw a rectangle and two lines
    - run milling
    - finish milling (restore ion beam current)

"""

def main():

    # connect to microscope
    microscope, settings = utils.setup_session(manufacturer="Demo", ip_address="localhost")

    # rectangle pattern
    rectangle_pattern = FibsemRectangleSettings(
        width = 10.0e-6,
        height = 10.0e-6,
        depth = 2.0e-6,
        rotation = 0.0,
        center_x = 0.0,
        center_y = 0.0,
    )

    # line pattern one
    line_pattern_01 = FibsemLineSettings(
        start_x = 0.0,
        start_y = 0.0,
        end_x = 10.0e-6,
        end_y = 10.0e-6,
        depth = 2.0e-6,
    )

    # line pattern two (mirror of line pattern one)
    line_pattern_02 = line_pattern_01
    line_pattern_02.end_y = -line_pattern_01.end_y

    logging.info(f"""\nMilling Pattern Example: """)

    logging.info(f"The current milling settings are: \n{settings.milling}")
    logging.info(f"The current rectangle pattern is \n{rectangle_pattern}")
    logging.info(f"The current line pattern one is \n{line_pattern_01}")
    logging.info(f"The current line pattern two is \n{line_pattern_02}")
    logging.info("---------------------------------- Milling ----------------------------------\n")
    # setup patterns in a list
    patterns = [rectangle_pattern, line_pattern_01, line_pattern_02]

    # setup milling
    milling.setup_milling(microscope, settings.milling)

    # draw patterns
    for pattern in patterns:
        milling.draw_pattern(microscope, pattern)

    # run milling
    milling.run_milling(microscope, settings.milling.milling_current, milling_voltage=settings.milling.milling_voltage)

    # finish milling
    milling.finish_milling(microscope, microscope.system.ion.beam.beam_current)


if __name__ == "__main__":
    main()